@@ -119,8 +119,7 @@ enum ability_type
     ABIL_SIF_MUNA_DIVINE_ENERGY,
     ABIL_SIF_MUNA_STOP_DIVINE_ENERGY,
     // Trog
-    ABIL_TROG_BURN_SPELLBOOKS = 1080,
-    ABIL_TROG_BERSERK,
+    ABIL_TROG_BERSERK = 1080,
     ABIL_TROG_REGEN_MR,
     ABIL_TROG_BROTHERS_IN_ARMS,
     // Elyvilon
