@@ -3,8 +3,7 @@
  * @brief Misc abyss specific functions.
 **/
 
-#ifndef ABYSS_H
-#define ABYSS_H
+#pragma once
 
 // When shifting areas in the abyss, shift the square containing player LOS
 // plus a little extra so that the player won't be disoriented by taking a
@@ -47,5 +46,3 @@ bool lugonu_corrupt_level(int power);
 void run_corruption_effects(int duration);
 void set_abyss_state(coord_def coord, uint32_t depth);
 void destroy_abyss();
-
-#endif
