@@ -53,6 +53,7 @@
 #include "prompt.h"
 #include "random.h"
 #include "religion.h"
+#include "scroller.h"
 #include "spl-clouds.h"
 #include "spl-goditem.h"
 #include "spl-miscast.h"
@@ -72,6 +73,8 @@
 #include "view.h"
 #include "xom.h"
 
+using namespace ui;
+
 // For information on deck structure, reference the comment near the beginning
 
 #define STACKED_KEY "stacked"
@@ -734,18 +737,6 @@ bool deck_deal()
     return true;
 }
 
-static void _redraw_stacked_cards(const vector<card_type>& draws,
-                                  unsigned int selected)
-{
-    for (unsigned int i = 0; i < draws.size(); ++i)
-    {
-        cgotoxy(1, i+2);
-        textcolour(selected == i ? WHITE : LIGHTGREY);
-        cprintf("%u - %s", i+1, card_name(draws[i]));
-        clear_to_end_of_line();
-    }
-}
-
 static bool _card_in_deck(card_type card, const deck_archetype *pdeck)
 {
     return any_of(pdeck->begin(), pdeck->end(),
@@ -793,30 +784,84 @@ static void _describe_cards(vector<card_type> cards)
 {
     ASSERT(!cards.empty());
 
-#ifdef USE_TILE_WEB
-    tiles_crt_control show_as_menu(CRT_MENU, "describe_cards");
-#endif
+    auto scroller = make_shared<Scroller>();
+    auto vbox = make_shared<Box>(Widget::VERT);
 
+#ifdef USE_TILE_WEB
+    tiles.json_open_object();
+    tiles.json_open_array("cards");
+#endif
+    bool seen[NUM_CARDS] = {0};
     ostringstream data;
     bool first = true;
     for (card_type card : cards)
     {
+        if (seen[card])
+            continue;
+        seen[card] = true;
+
         string name = card_name(card);
         string desc = getLongDescription(name + " card");
         if (desc.empty())
-            desc = "No description found.";
+            desc = "No description found.\n";
+        string decks = which_decks(card);
 
         name = uppercase_first(name);
-        if (first)
-            first = false;
-        else
-            data << "\n";
-        data << "<w>" << name << "</w>\n"
-             << get_linebreak_string(desc, get_number_of_cols() - 1)
-             << "\n" << which_decks(card) << "\n";
+        desc = desc + decks;
+
+    auto title_hbox = make_shared<Box>(Widget::HORZ);
+#ifdef USE_TILE
+        auto icon = make_shared<Image>();
+        icon->set_tile(tile_def(TILE_MISC_CARD, TEX_DEFAULT));
+        title_hbox->add_child(move(icon));
+#endif
+        auto title = make_shared<Text>(formatted_string(name, WHITE));
+        title->set_margin_for_crt({0, 0, 0, 0});
+        title->set_margin_for_sdl({0, 0, 0, 10});
+        title_hbox->add_child(move(title));
+        title_hbox->align_items = Widget::CENTER;
+        title_hbox->set_margin_for_crt({first ? 0 : 1, 0, 1, 0});
+        title_hbox->set_margin_for_sdl({first ? 0 : 20, 0, 20, 0});
+        vbox->add_child(move(title_hbox));
+
+        auto text = make_shared<Text>(desc);
+        text->wrap_text = true;
+        vbox->add_child(move(text));
+
+#ifdef USE_TILE_WEB
+        tiles.json_open_object();
+        tiles.json_write_string("name", name);
+        tiles.json_write_string("desc", desc);
+        tiles.json_close_object();
+#endif
+        first = false;
     }
-    formatted_scroller fs(0, data.str());
-    fs.show();
+
+#ifdef USE_TILE_LOCAL
+    vbox->max_size()[0] = tiles.get_crt_font()->char_width()*80;
+#endif
+
+    scroller->set_child(move(vbox));
+    auto popup = make_shared<ui::Popup>(scroller);
+
+    bool done = false;
+    popup->on(Widget::slots.event, [&done, &scroller](wm_event ev) {
+        if (ev.type != WME_KEYDOWN)
+            return false;
+        done = !scroller->on_event(ev);
+        return true;
+    });
+
+#ifdef USE_TILE_WEB
+    tiles.json_close_array();
+    tiles.push_ui_layout("describe-cards", 0);
+#endif
+
+    ui::run_layout(move(popup), done);
+
+#ifdef USE_TILE_WEB
+    tiles.pop_ui_layout();
+#endif
 }
 
 // Stack a deck: look at the next five cards, put them back in any
@@ -855,6 +900,53 @@ bool deck_stack()
     return true;
 }
 
+class StackFiveMenu : public Menu
+{
+    virtual bool process_key(int keyin) override;
+    vector<card_type>& draws;
+    vector<uint8_t>& flags;
+public:
+    StackFiveMenu(vector<card_type>& d, vector<uint8_t>& f)
+        : Menu(MF_NOSELECT | MF_ALWAYS_SHOW_MORE), draws(d), flags(f) {};
+};
+
+bool StackFiveMenu::process_key(int keyin)
+{
+    if (keyin == CK_ENTER)
+    {
+        formatted_string old_more = more;
+        set_more(formatted_string::parse_string(
+                "Are you done? (press y or Y to confirm)"));
+        if (yesno(nullptr, true, 'n', false, false, true))
+            return false;
+        set_more(old_more);
+    }
+    else if (keyin == '?')
+        _describe_cards(draws);
+    else if (keyin >= '1' && keyin <= '0' + static_cast<int>(draws.size()))
+    {
+        const unsigned int i = keyin - '1';
+        for (unsigned int j = 0; j < items.size(); j++)
+            if (items[j]->selected())
+            {
+                swap(draws[i], draws[j]);
+                swap(flags[i], flags[j]);
+                swap(items[i]->text, items[j]->text);
+                items[j]->colour = LIGHTGREY;
+                select_item_index(i, 0, false); // this also updates the item
+                select_item_index(j, 0, false);
+                return true;
+            }
+        items[i]->colour = WHITE;
+        select_item_index(i, 1, false);
+    }
+    else if (keyin == CK_ESCAPE)
+        return !crawl_state.seen_hups;
+    else
+        Menu::process_key(keyin);
+    return true;
+}
+
 bool stack_five(int slot)
 {
     item_def& deck(you.inv[slot]);
@@ -864,10 +956,6 @@ bool stack_five(int slot)
     const int num_cards    = cards_in_deck(deck);
     const int num_to_stack = (num_cards < 5 ? num_cards : 5);
 
-#ifdef USE_TILE_WEB
-    tiles_crt_control show_as_menu(CRT_MENU, "deck_stack");
-#endif
-
     vector<card_type> draws;
     vector<uint8_t>   flags;
     for (int i = 0; i < num_cards; ++i)
@@ -887,71 +975,23 @@ bool stack_five(int slot)
     deck.used_count = -num_to_stack;
     props[STACKED_KEY] = true;
     you.wield_change = true;
-    bool done = true;
 
-    if (draws.size() > 1)
+    StackFiveMenu menu(draws, flags);
+    MenuEntry *const title = new MenuEntry("Select two cards to swap them:", MEL_TITLE);
+    menu.set_title(title);
+    for (unsigned int i = 0; i < draws.size(); i++)
     {
-        bool need_prompt_redraw = true;
-        unsigned int selected = draws.size();
-        while (true)
-        {
-            if (need_prompt_redraw)
-            {
-                clrscr();
-                cgotoxy(1,1);
-                textcolour(WHITE);
-                cprintf("Press a digit to select a card, then another digit "
-                        "to swap it.");
-                cgotoxy(1,10);
-                cprintf("Press ? for the card descriptions, or Enter to "
-                        "accept.");
-
-                _redraw_stacked_cards(draws, selected);
-                need_prompt_redraw = false;
-            }
-
-            // Hand-hacked implementation, instead of using Menu. Oh well.
-            const int c = getchk();
-            if (c == CK_ENTER)
-            {
-                cgotoxy(1,11);
-                textcolour(LIGHTGREY);
-                cprintf("Are you done? (press y or Y to confirm)");
-                if (toupper(getchk()) == 'Y')
-                    break;
-
-                cgotoxy(1,11);
-                clear_to_end_of_line();
-                continue;
-            }
-
-            if (c == '?')
-            {
-                _describe_cards(draws);
-                need_prompt_redraw = true;
-            }
-            else if (c >= '1' && c <= '0' + static_cast<int>(draws.size()))
-            {
-                const unsigned int new_selected = c - '1';
-                if (selected < draws.size())
-                {
-                    swap(draws[selected], draws[new_selected]);
-                    swap(flags[selected], flags[new_selected]);
-                    selected = draws.size();
-                }
-                else
-                    selected = new_selected;
-
-                _redraw_stacked_cards(draws, selected);
-            }
-            else if (c == CK_ESCAPE && crawl_state.seen_hups)
-            {
-                done = false;
-                break; // continue on game restore
-            }
-        }
-        redraw_screen();
+        MenuEntry * const entry = new MenuEntry(card_name(draws[i]), MEL_ITEM, 1, '1'+i);
+#ifdef USE_TILE
+        entry->add_tile(tile_def(TILE_MISC_CARD, TEX_DEFAULT));
+#endif
+        menu.add_entry(entry);
     }
+    menu.set_more(formatted_string::parse_string(
+                "<lightgrey>Press <w>?</w> for the card descriptions"
+                " or <w>Enter</w> to accept."));
+    menu.show();
+
     for (unsigned int i = 0; i < draws.size(); ++i)
     {
         _push_top_card(deck, draws[draws.size() - 1 - i],
@@ -961,7 +1001,7 @@ bool stack_five(int slot)
     _check_buggy_deck(deck);
     you.wield_change = true;
 
-    return done;
+    return !crawl_state.seen_hups;
 }
 
 // Draw the next three cards, discard two and pick one.
@@ -1166,7 +1206,6 @@ static void _suppressed_card_message(god_type god, conduct_type done)
         case DID_EVIL: forbidden_act = "evil"; break;
         case DID_CHAOS: forbidden_act = "chaotic"; break;
         case DID_HASTY: forbidden_act = "hasty"; break;
-        case DID_FIRE: forbidden_act = "fiery"; break;
 
         default: forbidden_act = "buggy"; break;
     }
@@ -1642,23 +1681,10 @@ static void _elements_card(int power, deck_rarity_type rarity)
     };
 
     int start = random2(ARRAYSZ(element_list));
-
     for (int i = 0; i < 3; ++i)
     {
-        monster_type mons_type = element_list[start % ARRAYSZ(element_list)][power_level];
-        monster hackmon;
-
-        hackmon.type = mons_type;
-        mons_load_spells(hackmon);
-
-        if (you_worship(GOD_DITHMENOS) && mons_is_fiery(hackmon))
-        {
-            _suppressed_card_message(you.religion, DID_FIRE);
-            start++;
-            continue;
-        }
-
-        _friendly(mons_type, power_level + 2);
+        _friendly(element_list[start % ARRAYSZ(element_list)][power_level],
+                  power_level + 2);
         start++;
     }
 
@@ -1848,16 +1874,13 @@ static void _cloud_card(int power, deck_rarity_type rarity)
 
         switch (power_level)
         {
-            case 0: cloudy = (you_worship(GOD_SHINING_ONE) || !one_chance_in(5))
-                              ? CLOUD_MEPHITIC : CLOUD_POISON;
+            case 0: cloudy = !one_chance_in(5) ? CLOUD_MEPHITIC : CLOUD_POISON;
                     break;
 
-            case 1: cloudy = (you_worship(GOD_DITHMENOS) || coinflip())
-                              ? CLOUD_COLD : CLOUD_FIRE;
+            case 1: cloudy = coinflip() ? CLOUD_COLD : CLOUD_FIRE;
                     break;
 
-            case 2: cloudy = (is_good_god(you.religion) || coinflip())
-                              ? CLOUD_ACID: CLOUD_MIASMA;
+            case 2: cloudy = coinflip() ? CLOUD_ACID: CLOUD_MIASMA;
                     break;
 
             default: cloudy = CLOUD_DEBUGGING;
@@ -1941,12 +1964,6 @@ static void _degeneration_card(int power, deck_rarity_type rarity)
 {
     const int power_level = _get_power_level(power, rarity);
 
-    if (you_worship(GOD_ZIN))
-    {
-        _suppressed_card_message(you.religion, DID_CHAOS);
-        return;
-    }
-
     if (!apply_visible_monsters([power_level](monster& mons)
            {
                if (mons.wont_attack() || !mons_is_threatening(mons))
