@@ -42,6 +42,7 @@
 #include "libutil.h"
 #include "macro.h"
 #include "message.h"
+#include "mon-act.h"
 #include "mon-behv.h"
 #include "mon-tentacle.h"
 #include "mon-util.h"
@@ -728,7 +729,7 @@ void JewelleryOnDelay::finish()
 #ifdef USE_SOUND
     parse_sound(WEAR_JEWELLERY_SOUND);
 #endif
-    puton_ring(jewellery.link, false);
+    puton_ring(jewellery.link, false, false);
 }
 
 void ArmourOnDelay::finish()
@@ -760,6 +761,11 @@ void ArmourOnDelay::finish()
     check_item_hint(armour, old_talents);
 }
 
+bool ArmourOffDelay::invalidated()
+{
+    return !armour.defined();
+}
+
 void ArmourOffDelay::finish()
 {
     const equipment_type slot = get_armour_slot(armour);
@@ -1238,7 +1244,7 @@ static inline bool _monster_warning(activity_interrupt_type ai,
         {
             yell(mon);
         }
-        mon->seen_context = SC_JUST_SEEN;
+        mons_set_just_seen(mon);
     }
 
     if (crawl_state.game_is_hints())
