@@ -88,11 +88,13 @@ struct direction_chooser_args
     int range;
     bool just_looking;
     bool needs_path;
+    bool unrestricted; // for wizmode
     confirm_prompt_type self;
     const char *target_prefix;
     string top_prompt;
     targeting_behaviour *behaviour;
     bool show_floor_desc;
+    bool show_boring_feats;
     desc_filter get_desc_func;
     coord_def default_place;
 
@@ -103,10 +105,12 @@ struct direction_chooser_args
         range(-1),
         just_looking(false),
         needs_path(true),
+        unrestricted(false),
         self(CONFIRM_PROMPT),
         target_prefix(nullptr),
         behaviour(nullptr),
         show_floor_desc(false),
+        show_boring_feats(true),
         get_desc_func(nullptr),
         default_place(0, 0) {}
 };
@@ -246,6 +250,7 @@ private:
     string top_prompt;          // Shown at the top of the message window
     targeting_behaviour *behaviour; // Can be nullptr for default
     bool show_floor_desc;       // Describe the floor of the current target
+    bool show_boring_feats;
     targeter *hitfunc;         // Determine what would be hit.
     coord_def default_place;    // Start somewhere other than you.pos()?
 
@@ -270,6 +275,8 @@ private:
     // Default behaviour, saved across instances.
     static targeting_behaviour stock_behaviour;
 
+    bool unrestricted;
+
 public:
     // TODO: fix the weird behavior that led to this hack
     bool needs_path;            // Determine a ray while we're at it?
