@@ -49,6 +49,7 @@
 #include "god-passive.h"
 #include "hints.h"
 #include "initfile.h"
+#include "item-name.h"
 #include "items.h"
 #include "jobs.h"
 #include "kills.h"
@@ -93,6 +94,29 @@
 #define F_OK 0
 #endif
 
+#define BONES_DIAGNOSTICS (defined(WIZARD) || defined(DEBUG_BONES) || defined(DEBUG_DIAGNOSTICS))
+
+#ifdef BONES_DIAGNOSTICS
+/// show diagnostics following a wizard command, even if not a debug build
+static void _ghost_dprf(const char *format, ...)
+{
+    va_list argp;
+    va_start(argp, format);
+
+    const bool wiz_cmd = (crawl_state.prev_cmd == CMD_WIZARD);
+    if (wiz_cmd)
+        do_message_print(MSGCH_ERROR, 0, true, false, format, argp);
+#ifdef DEBUG_DIAGNOSTICS
+    else
+        do_message_print(MSGCH_DIAGNOSTICS, 0, false, false, format, argp);
+#endif
+
+    va_end(argp);
+}
+#else
+# define _ghost_dprf(...) ((void)0)
+#endif
+
 static void _save_level(const level_id& lid);
 
 static bool _ghost_version_compatible(reader &ghost_reader);
@@ -434,6 +458,77 @@ static vector<string> _get_base_dirs()
     return bases;
 }
 
+void validate_basedirs()
+{
+    // TODO: could use this to pick a single data directory?
+    vector<string> bases(_get_base_dirs());
+    bool found;
+
+    // there are a few others, but this should be enough to minimally run something
+    const vector<string> data_subfolders =
+    {
+#ifdef CLUA_BINDINGS
+        "clua",
+#endif
+        "database",
+        "defaults",
+        "des",
+        "descript",
+        "dlua"
+#ifdef USE_TILE_LOCAL
+        , "tiles"
+#endif
+    };
+
+    for (const string &d : bases)
+    {
+        if (dir_exists(d))
+        {
+            bool everything = true;
+            bool something = false;
+            for (auto subdir : data_subfolders)
+            {
+                if (dir_exists(d + subdir))
+                    something = true;
+                else
+                    everything = false;
+            }
+            if (everything)
+            {
+                mprf(MSGCH_PLAIN, "Data directory '%s' found.", d.c_str());
+                found = true;
+            }
+            else if (something)
+            {
+                // give an error for this case because this incomplete data
+                // directory will be checked before others, possibly leading
+                // to a weird mix of data files.
+                if (!found)
+                {
+                    mprf(MSGCH_ERROR,
+                        "Incomplete or corrupted data directory '%s'",
+                                d.c_str());
+                }
+            }
+        }
+    }
+
+    // can't proceed if nothing complete was found.
+    if (!found)
+    {
+        string err = "Missing DCSS data directory; tried: \n";
+        for (const string &d : bases)
+            err += d + ", ";
+        if (err.size() > 2)
+        {
+            err.pop_back();
+            err.pop_back();
+        }
+
+        end(1, false, "%s", err.c_str());
+    }
+}
+
 string datafile_path(string basename, bool croak_on_fail, bool test_base_path,
                      bool (*thing_exists)(const string&))
 {
@@ -911,7 +1006,7 @@ static void _grab_followers()
     monster* dowan = nullptr;
     monster* duvessa = nullptr;
 
-    // Handle nearby ghosts.
+    // Handle some hacky cases
     for (adjacent_iterator ai(you.pos()); ai; ++ai)
     {
         monster* fol = monster_at(*ai);
@@ -932,14 +1027,6 @@ static void _grab_followers()
             if (mons_class_can_use_stairs(fol->type))
                 non_stair_using_summons++;
         }
-
-        if (fol->type == MONS_PLAYER_GHOST
-            && fol->hit_points < fol->max_hit_points / 2)
-        {
-            if (fol->visible_to(&you))
-                mpr("The ghost fades into the shadows.");
-            monster_teleport(fol, true);
-        }
     }
 
     // Deal with Dowan and Duvessa here.
@@ -1177,8 +1264,6 @@ static void _make_level(dungeon_feature_type stair_taken,
     _clear_env_map();
     builder(true, stair_type);
 
-    if (ghost_demon::ghost_eligible() && one_chance_in(3))
-        load_ghosts(ghost_demon::max_ghosts_per_level(env.absdepth0), true);
     env.turns_on_level = 0;
     // sanctuary
     env.sanctuary_pos  = coord_def(-1, -1);
@@ -1688,12 +1773,8 @@ void save_game(bool leave_game, const char *farewellmsg)
     // so Valgrind doesn't complain.
     _save_game_exit();
 
-    // TODO: just call game_ended?
-    if (crawl_should_restart(game_exit::save) && !crawl_state.seen_hups)
-        throw game_ended_condition(game_exit::save);
-
-    end(0, false, farewellmsg? "%s" : "See you soon, %s!",
-        farewellmsg? farewellmsg : you.your_name.c_str());
+    game_ended(game_exit::save, farewellmsg ? farewellmsg
+                                : "See you soon, " + you.your_name + "!");
 }
 
 // Saves the game without exiting.
@@ -1704,13 +1785,38 @@ void save_game_state()
         save_game(true);
 }
 
-static string _make_ghost_filename()
+static string _make_ghost_filename(bool store=false)
 {
-    return "bones."
-           + replace_all(level_id::current().describe(), ":", "-");
+    // Only use level-numbered bones files for places where players die a lot.
+    // For the permastore, go even coarser (just D and Lair use level numbers).
+    // n.b. some branches here may not currently generate ghosts.
+    // TODO: further adjustments? Make Zot coarser?
+    const bool with_number = store ? player_in_branch(BRANCH_DUNGEON) ||
+                                     player_in_branch(BRANCH_LAIR)
+                                   : !(player_in_branch(BRANCH_ZIGGURAT) ||
+                                       player_in_branch(BRANCH_CRYPT) ||
+                                       player_in_branch(BRANCH_TOMB) ||
+                                       player_in_branch(BRANCH_ABYSS) ||
+                                       player_in_branch(BRANCH_SLIME));
+    // Players die so rarely in hell in practice that it doesn't even make
+    // sense to have per-hell bones. (Maybe vestibule should be separate?)
+    const string level_desc = player_in_hell() ? "Hells" :
+        replace_all(level_id::current().describe(false, with_number), ":", "-");
+    return string("bones.") + (store ? "store." : "") + level_desc;
 }
 
-#define BONES_DIAGNOSTICS (defined(WIZARD) || defined(DEBUG_BONES) | defined(DEBUG_DIAGNOSTICS))
+static string _bones_permastore_file()
+{
+    return _get_bonefile_directory() + _make_ghost_filename(true);
+}
+
+// Bones files
+//
+// There are two kinds of bones files: temporary bones files and the
+// permastore. Temporary bones files are ephemeral: ghosts will be reused only
+// if they are on the floor where the player dies. The permastore is a more
+// permanent stock of ghosts (per level) to use as a backup in case the
+// temporary bones files are depleted.
 
 /**
  * Lists all bonefiles for the current level.
@@ -1727,12 +1833,15 @@ static vector<string> _list_bones()
     vector<string> bonefiles;
     for (const auto &filename : filenames)
         if (starts_with(filename, underscored_filename))
+        {
             bonefiles.push_back(bonefile_dir + filename);
+            _ghost_dprf("bonesfile %s", (bonefile_dir + filename).c_str());
+        }
 
     string old_bonefile = _get_old_bonefile_directory() + base_filename;
     if (access(old_bonefile.c_str(), F_OK) == 0)
     {
-        dprf("Found old bonefile %s", old_bonefile.c_str());
+        _ghost_dprf("Found old bonefile %s", old_bonefile.c_str());
         bonefiles.push_back(old_bonefile);
     }
 
@@ -1752,61 +1861,143 @@ static string _find_ghost_file()
     return bonefiles[ui_random(bonefiles.size())];
 }
 
-static vector<ghost_demon> _load_ghost_vec(bool creating_level, bool wiz_cmd)
+static vector<ghost_demon> _load_bones_file(string ghost_filename)
 {
     vector<ghost_demon> result;
 
-    const string ghost_filename = _find_ghost_file();
     if (ghost_filename.empty())
-    {
-        if (wiz_cmd && !creating_level)
-            mprf(MSGCH_PROMPT, "No ghost files for this level.");
         return result; // no such ghost.
-    }
 
     reader inf(ghost_filename);
     if (!inf.valid())
     {
-        if (wiz_cmd && !creating_level)
-            mprf(MSGCH_PROMPT, "Ghost file invalidated before read.");
+        _ghost_dprf("Ghost file '%s' invalid before read.", ghost_filename.c_str());
         return result;
     }
 
     inf.set_safe_read(true); // don't die on 0-byte bones
-    if (_ghost_version_compatible(inf))
+    if (!_ghost_version_compatible(inf))
     {
-        try
-        {
-            result = tag_read_ghosts(inf);
-            inf.fail_if_not_eof(ghost_filename);
-        }
-        catch (short_read_exception &short_read)
-        {
-            mprf(MSGCH_ERROR, "Broken bones file: %s",
-                 ghost_filename.c_str());
-        }
+        string error = "Incompatible bones file: " + ghost_filename;
+        throw corrupted_save(error);
+    }
+    try
+    {
+        result = tag_read_ghosts(inf);
+        inf.fail_if_not_eof(ghost_filename);
+    }
+    catch (short_read_exception &short_read)
+    {
+        string error = "Broken bones file: " + ghost_filename;
+        throw corrupted_save(error);
     }
     inf.close();
 
-    // Remove bones file - ghosts are hardly permanent.
+    if (!debug_check_ghosts(result))
+    {
+        string error = "Bones file is buggy: " + ghost_filename;
+        throw corrupted_save(error);
+    }
+
+    return result;
+}
+
+static vector<ghost_demon> _load_ephemeral_ghosts()
+{
+    vector<ghost_demon> results;
+
+    string ghost_filename = _find_ghost_file();
+    if (ghost_filename.empty())
+    {
+        _ghost_dprf("%s", "No ghost files for this level.");
+        return results; // no such ghost.
+    }
+
+    try
+    {
+        results = _load_bones_file(ghost_filename);
+    }
+    catch (corrupted_save &err)
+    {
+        mprf(MSGCH_ERROR, "%s", err.what());
+    }
+
     if (unlink(ghost_filename.c_str()) != 0)
     {
         mprf(MSGCH_ERROR, "Failed to unlink bones file: %s",
                 ghost_filename.c_str());
     }
+    return results;
+}
 
-    if (!debug_check_ghosts(result))
+static vector<ghost_demon> _load_permastore_ghosts()
+{
+    string ghost_filename = _bones_permastore_file();
+    vector<ghost_demon> results;
+    try
     {
-        mprf(MSGCH_DIAGNOSTICS,
-             "Refusing to load buggy ghost from file \"%s\"!",
-             ghost_filename.c_str());
+        results = _load_bones_file(ghost_filename);
+    }
+    catch (corrupted_save &err)
+    {
+        mprf(MSGCH_ERROR, "%s", err.what());
+        string report;
+        // if we get to this point the permastore is unreadable and needs to
+        // be scrapped
+        if (unlink(ghost_filename.c_str()) != 0)
+            report = "Failed to unlink bad ghost permastore";
+        else
+            report = "Clearing bad ghost permastore";
+        mprf(MSGCH_ERROR, "%s: %s", report.c_str(), ghost_filename.c_str());
+    }
+    // otherwise, don't do anything to the permastore
+    return results;
+}
 
-        result.clear();
-        return result;
+/**
+ * Attempt to fill in a monster based on bones files.
+ *
+ * @param mons the monster to fill in
+ *
+ * @return whether there was a saved ghost that could be used.
+ */
+bool define_ghost_from_bones(monster& mons)
+{
+    bool used_permastore = false;
+
+    vector<ghost_demon> loaded_ghosts = _load_ephemeral_ghosts();
+    if (loaded_ghosts.empty())
+    {
+        loaded_ghosts = _load_permastore_ghosts();
+        if (loaded_ghosts.empty())
+            return false;
+        used_permastore = true;
     }
 
-    return result;
+    int place_i = random2(loaded_ghosts.size());
+    _ghost_dprf("Loaded ghost file with %u ghost(s), placing %s",
+         (unsigned int)loaded_ghosts.size(), loaded_ghosts[place_i].name.c_str());
 
+    mons.set_ghost(loaded_ghosts[place_i]);
+    mons.type = MONS_PLAYER_GHOST;
+    mons.ghost_init(false);
+
+    if (!mons.alive())
+        mprf(MSGCH_ERROR, "Placed ghost is not alive.");
+    else if (mons.type != MONS_PLAYER_GHOST)
+    {
+        mprf(MSGCH_ERROR, "Placed ghost is not MONS_PLAYER_GHOST, but %s",
+             mons.name(DESC_PLAIN, true).c_str());
+    }
+
+    if (!used_permastore)
+    {
+        loaded_ghosts.erase(loaded_ghosts.begin());
+
+        if (!loaded_ghosts.empty())
+            save_ghosts(loaded_ghosts);
+    }
+    return true;
 }
 
 /**
@@ -1819,43 +2010,28 @@ static vector<ghost_demon> _load_ghost_vec(bool creating_level, bool wiz_cmd)
  */
 bool load_ghosts(int max_ghosts, bool creating_level)
 {
-    const bool wiz_cmd = (crawl_state.prev_cmd == CMD_WIZARD);
-
     ASSERT(you.transit_stair == DNGN_UNSEEN || creating_level);
     ASSERT(!you.entering_level || creating_level);
     ASSERT(!creating_level
            || (you.entering_level && you.transit_stair != DNGN_UNSEEN));
     // Only way to load a ghost without creating a level is via a wizard
     // command.
-    ASSERT(creating_level || wiz_cmd);
+    ASSERT(creating_level || (crawl_state.prev_cmd == CMD_WIZARD));
 
-#if !defined(DEBUG) && !defined(WIZARD)
-    UNUSED(creating_level);
+#ifdef BONES_DIAGNOSTICS
+    // this is pretty hacky, but arguably cleaner than what it is replacing.
+    // The effect is to show bones diagnostic messages on wizmode builds during
+    // level building
+    unwind_var<command_type> last_cmd(crawl_state.prev_cmd, creating_level ?
+        CMD_WIZARD : crawl_state.prev_cmd);
 #endif
 
-#ifdef BONES_DIAGNOSTICS
-    const bool do_diagnostics =
-#  if defined(DEBUG_BONES) || defined(DEBUG_DIAGNOSTICS)
-        true
-#  elif defined(WIZARD)
-        !creating_level
-#  else // Can't happen currently
-        false
-#  endif
-        ;
-#endif // BONES_DIAGNOSTICS
+    vector<ghost_demon> loaded_ghosts = _load_ephemeral_ghosts();
 
-    vector<ghost_demon> loaded_ghosts = _load_ghost_vec(creating_level, wiz_cmd);
-
-#ifdef BONES_DIAGNOSTICS
-    if (do_diagnostics)
-    {
-        mprf(MSGCH_DIAGNOSTICS, "Loaded ghost file with %u ghost(s), will attempt to place %d of them",
+    _ghost_dprf("Loaded ghost file with %u ghost(s), will attempt to place %d of them",
              (unsigned int)loaded_ghosts.size(), max_ghosts);
-    }
 
-    bool          ghost_errors    = false;
-#endif
+    bool ghost_errors = false;
 
     max_ghosts = max_ghosts <= 0 ? loaded_ghosts.size()
                                  : min(max_ghosts, (int) loaded_ghosts.size());
@@ -1876,35 +2052,29 @@ bool load_ghosts(int max_ghosts, bool creating_level)
         loaded_ghosts.erase(loaded_ghosts.begin());
         placed_ghosts++;
 
-#ifdef BONES_DIAGNOSTICS
-        if (do_diagnostics)
+        if (!mons->alive())
         {
-            if (!mons->alive())
-            {
-                mprf(MSGCH_DIAGNOSTICS, "Placed ghost is not alive.");
-                ghost_errors = true;
-            }
-            else if (mons->type != MONS_PLAYER_GHOST)
-            {
-                mprf(MSGCH_DIAGNOSTICS,
-                     "Placed ghost is not MONS_PLAYER_GHOST, but %s",
-                     mons->name(DESC_PLAIN, true).c_str());
-                ghost_errors = true;
-            }
+            _ghost_dprf("Placed ghost is not alive.");
+            ghost_errors = true;
+        }
+        else if (mons->type != MONS_PLAYER_GHOST)
+        {
+            _ghost_dprf("Placed ghost is not MONS_PLAYER_GHOST, but %s",
+                 mons->name(DESC_PLAIN, true).c_str());
+            ghost_errors = true;
         }
-#endif
     }
 
-#ifdef BONES_DIAGNOSTICS
-    if (do_diagnostics && placed_ghosts < max_ghosts)
+    if (placed_ghosts < max_ghosts)
     {
-        mprf(MSGCH_DIAGNOSTICS, "Unable to place %u ghost(s)",
-             max_ghosts - placed_ghosts);
+        _ghost_dprf("Unable to place %u ghost(s)", max_ghosts - placed_ghosts);
         ghost_errors = true;
     }
+#ifdef BONES_DIAGNOSTICS
     if (ghost_errors)
         more();
 #endif
+
     // resave any unused ghosts
     if (!loaded_ghosts.empty())
         save_ghosts(loaded_ghosts);
@@ -1918,12 +2088,10 @@ static bool _restore_game(const string& filename)
     if (Options.no_save)
         return false;
 
-#ifdef USE_TILE_WEB
-    // a more before the player is loaded will crash when it tries to send
-    // enough information to the webtiles client to render the display.
-    // TODO: is there a good way to do any mores in load_messages?
+    // In webtiles, a more before the player is loaded will crash when it tries
+    // to send enough information to the webtiles client to render the display.
+    // This is just cosmetic for other build targets.
     unwind_bool save_more(crawl_state.show_more_prompt, false);
-#endif
 
     clear_message_store();
 
@@ -1944,21 +2112,29 @@ static bool _restore_game(const string& filename)
             you.save = 0;
             return false;
         }
-        fail("Cannot load an incompatible save from version %s",
-             you.prev_save_version.c_str());
+        crawl_state.default_startup_name = you.your_name; // for main menu
+        you.save->abort();
+        delete you.save;
+        you.save = 0;
+        game_ended(game_exit::abort,
+            you.your_name + " is from an incompatible version and can't be loaded.");
     }
 
+    crawl_state.default_startup_name = you.your_name; // for main menu
+
     if (numcmp(you.prev_save_version.c_str(), Version::Long, 2) == -1
         && version_is_stable(you.prev_save_version.c_str()))
     {
-        if (!yesno("This game comes from a previous release of Crawl. If you "
-                   "load it now, you won't be able to go back. Continue?",
-                   true, 'n'))
+        if (!yesno(("This game comes from a previous release of Crawl (" +
+                    you.prev_save_version + ").\n\nIf you load it now,"
+                    " you won't be able to go back. Continue?").c_str(),
+                    true, 'n'))
         {
             you.save->abort(); // don't even rewrite the header
             delete you.save;
             you.save = 0;
-            end(0, false, "Please reinstall the stable version then.\n");
+            game_ended(game_exit::abort, "Please use version " +
+                you.prev_save_version + " to load " + you.your_name + " then.");
         }
     }
 
@@ -2152,7 +2328,9 @@ static bool _convert_obsolete_species()
             you.save->abort(); // don't even rewrite the header
             delete you.save;
             you.save = 0;
-            end(0, false, "Please load the save in an earlier version if you want to keep it as a Lava Orc.\n");
+            game_ended(game_exit::abort,
+                "Please load the save in an earlier version "
+                "if you want to keep it as a Lava Orc.");
         }
         change_species_to(SP_HILL_ORC);
         // No need for conservation
@@ -2172,7 +2350,9 @@ static bool _convert_obsolete_species()
             you.save->abort(); // don't even rewrite the header
             delete you.save;
             you.save = 0;
-            end(0, false, "Please load the save in an earlier version if you want to keep it as a Djinni.\n");
+            game_ended(game_exit::abort,
+                "Please load the save in an earlier version "
+                "if you want to keep it as a Djinni.");
         }
         change_species_to(SP_VINE_STALKER);
         you.magic_contamination = 0;
@@ -2357,17 +2537,17 @@ static bool _ghost_version_compatible(reader &inf)
  **/
 static FILE* _make_bones_file(string * return_gfilename)
 {
-
     const string bone_dir = _get_bonefile_directory();
-    const string base_filename = _make_ghost_filename();
+    const string base_filename = _make_ghost_filename(false);
+
     for (int i = 0; i < GHOST_LIMIT; i++)
     {
         const string g_file_name = make_stringf("%s%s_%d", bone_dir.c_str(),
                                                 base_filename.c_str(), i);
-        FILE *gfil = lk_open_exclusive(g_file_name);
+        FILE *ghost_file = lk_open_exclusive(g_file_name);
         // need to check file size, so can't open 'wb' - would truncate!
 
-        if (!gfil)
+        if (!ghost_file)
         {
             dprf("Could not open %s", g_file_name.c_str());
             continue;
@@ -2376,12 +2556,75 @@ static FILE* _make_bones_file(string * return_gfilename)
         dprf("found %s", g_file_name.c_str());
 
         *return_gfilename = g_file_name;
-        return gfil;
+        return ghost_file;
     }
 
     return nullptr;
 }
 
+#define GHOST_PERMASTORE_SIZE 10
+#define GHOST_PERMASTORE_REPLACE_CHANCE 5
+
+static bool _update_permastore(const vector<ghost_demon> &ghosts)
+{
+    if (ghosts.empty())
+        return false;
+
+    // this read is not locked...
+    vector<ghost_demon> permastore = _load_permastore_ghosts();
+
+    bool rewrite = false;
+    unsigned int i = 0;
+    while (permastore.size() < GHOST_PERMASTORE_SIZE && i < ghosts.size())
+    {
+        // TODO: heuristics to make this as distinct as possible; maybe
+        // create a new name?
+        permastore.push_back(ghosts[i]);
+#ifdef DGAMELAUNCH
+        // randomize name for online play
+        permastore.back().name = make_name();
+#endif
+        i++;
+        rewrite = true;
+    }
+    if (i > 0)
+        _ghost_dprf("Permastoring %d ghosts", i);
+    if (!rewrite && x_chance_in_y(GHOST_PERMASTORE_REPLACE_CHANCE, 100))
+    {
+        int rewrite_i = random2(permastore.size());
+        permastore[rewrite_i] = ghosts[0];
+#ifdef DGAMELAUNCH
+        permastore[rewrite_i].name = make_name();
+#endif
+        rewrite = true;
+    }
+
+    if (rewrite)
+    {
+        string permastore_file = _bones_permastore_file();
+        FILE *ghost_file = lk_open("wb", permastore_file);
+
+        if (!ghost_file)
+        {
+            // this will fail silently if the lock fails, seems safest
+            // TODO: better lock system for servers?
+            _ghost_dprf("Could not open ghost permastore: %s",
+                                                    permastore_file.c_str());
+            return false;
+        }
+
+        _ghost_dprf("Rewriting ghost permastore %s with %u ghosts",
+                    permastore_file.c_str(), (unsigned int) permastore.size());
+        writer outw(permastore_file, ghost_file);
+        _write_ghost_version(outw);
+        tag_write_ghosts(outw, permastore);
+
+        lk_close(ghost_file, permastore_file);
+        return true;
+    }
+    return false;
+}
+
 /**
  * Attempt to save all ghosts from the current level.
  *
@@ -2391,38 +2634,31 @@ static FILE* _make_bones_file(string * return_gfilename)
  * @param force   Forces ghost generation even in otherwise-disallowed levels.
  **/
 
-void save_ghosts(const vector<ghost_demon> &ghosts, bool force)
+void save_ghosts(const vector<ghost_demon> &ghosts, bool force, bool use_store)
 {
-#ifdef BONES_DIAGNOSTICS
-    const bool do_diagnostics =
-#  if defined(DEBUG_BONES) || defined(DEBUG_DIAGNOSTICS)
-        true
-#  elif defined(WIZARD)
-        you.wizard
-#  else // Can't happen currently
-        false
-#  endif
-        ;
-#endif // BONES_DIAGNOSTICS
-
+    // n.b. this is not called in the normal course of events for wizmode
+    // chars, so for debugging anything to do with deaths in wizmode, you will
+    // need to edit a conditional at the end of ouch.cc:ouch.
+    _ghost_dprf("Trying to save ghosts.");
     if (ghosts.empty())
     {
-#ifdef BONES_DIAGNOSTICS
-        if (do_diagnostics)
-            mprf(MSGCH_DIAGNOSTICS, "Could not find any ghosts for this level.");
-#endif
+        _ghost_dprf("Could not find any ghosts for this level to save.");
         return;
     }
 
     if (!force && !ghost_demon::ghost_eligible())
+    {
+        _ghost_dprf("No eligible ghosts.");
+        return;
+    }
+
+    // TODO: fill temp store with ghosts unused by this
+    if (_update_permastore(ghosts))
         return;
 
     if (_list_bones().size() >= static_cast<size_t>(GHOST_LIMIT))
     {
-#ifdef BONES_DIAGNOSTICS
-        if (do_diagnostics)
-            mprf(MSGCH_DIAGNOSTICS, "Too many ghosts for this level already!");
-#endif
+        _ghost_dprf("Too many ghosts for this level already!");
         return;
     }
 
@@ -2431,10 +2667,7 @@ void save_ghosts(const vector<ghost_demon> &ghosts, bool force)
 
     if (!ghost_file)
     {
-#ifdef BONES_DIAGNOSTICS
-        if (do_diagnostics)
-            mprf(MSGCH_DIAGNOSTICS, "Could not open file to save ghosts.");
-#endif
+        _ghost_dprf("Could not open file to save ghosts.");
         return;
     }
 
@@ -2445,10 +2678,7 @@ void save_ghosts(const vector<ghost_demon> &ghosts, bool force)
 
     lk_close(ghost_file, g_file_name);
 
-#ifdef BONES_DIAGNOSTICS
-    if (do_diagnostics)
-        mprf(MSGCH_DIAGNOSTICS, "Saved ghosts (%s).", g_file_name.c_str());
-#endif
+    _ghost_dprf("Saved ghosts (%s).", g_file_name.c_str());
 }
 
 ////////////////////////////////////////////////////////////////////////////
