@@ -85,7 +85,6 @@ bool vehumet_supports_spell(spell_type spell);
 
 void sif_do_channel_energy(int pow);
 
-bool trog_burn_spellbooks();
 void trog_do_trogs_hand(int power);
 void trog_remove_trogs_hand();
 
@@ -96,6 +95,7 @@ bool beogh_gift_item();
 bool beogh_resurrect();
 
 bool yred_injury_mirror();
+bool yred_can_enslave_soul(monster* mon);
 void yred_make_enslaved_soul(monster* mon, bool force_hostile = false);
 
 bool kiku_receive_corpses(int pow);
