@@ -174,13 +174,14 @@ formatted_string hints_abilities_info();
 string hints_skills_info();
 string hints_skill_training_info();
 string hints_skills_description_info();
+string hints_memorise_info();
 
 // Additional information for tutorial players.
 void check_item_hint(const item_def &item, unsigned int num_old_talents);
 string hints_describe_item(const item_def &item);
 void hints_inscription_info(string prompt);
 bool hints_pos_interesting(int x, int y);
-void hints_describe_pos(int x, int y);
+string hints_describe_pos(int x, int y);
 bool hints_monster_interesting(const monster* mons);
 string hints_describe_monster(const monster_info& mi, bool has_stat_desc);
 
