@@ -44,6 +44,7 @@
 #include "ouch.h"
 #include "place.h"
 #include "religion.h"
+#include "scroller.h"
 #include "skills.h"
 #include "state.h"
 #include "status.h"
@@ -57,6 +58,8 @@
 #include "unwind.h"
 #include "version.h"
 
+using namespace ui;
+
 #define SCORE_VERSION "0.1"
 
 // enough memory allocated to snarf in the scorefile entries
@@ -248,20 +251,21 @@ void hiscores_print_all(int display_count, int format)
 
 // Displays high scores using curses. For output to the console, use
 // hiscores_print_all.
-void hiscores_print_list(int display_count, int format, int newest_entry)
+string hiscores_print_list(int display_count, int format, int newest_entry)
 {
     unwind_bool scorefile_display(crawl_state.updating_scores, true);
+    string ret;
 
     FILE *scores;
     int i, total_entries;
 
     if (display_count <= 0)
-        return;
+        return "";
 
     // open highscore file (reading)
     scores = _hs_open("r", _score_file_name());
     if (scores == nullptr)
-        return;
+        return "";
 
     // read highscore file
     for (i = 0; i < SCORE_FILE_ENTRIES; i++)
@@ -275,8 +279,6 @@ void hiscores_print_list(int display_count, int format, int newest_entry)
     // close off
     _hs_close(scores, _score_file_name());
 
-    textcolour(LIGHTGREY);
-
     int start = newest_entry - display_count / 2;
 
     if (start + display_count > total_entries)
@@ -291,13 +293,17 @@ void hiscores_print_list(int display_count, int format, int newest_entry)
     {
         // check for recently added entry
         if (i == newest_entry)
-            textcolour(YELLOW);
+            ret += "<yellow>";
 
-        _hiscores_print_entry(*hs_list[i], i, format, cprintf);
+        _hiscores_print_entry(*hs_list[i], i, format, [&ret](const char *fmt, const char *s){
+            ret += string(s);
+        });
 
         if (i == newest_entry)
-            textcolour(LIGHTGREY);
+            ret += "<lightgrey>";
     }
+
+    return ret;
 }
 
 static void _add_hiscore_row(MenuScroller* scroller, scorefile_entry& se, int id)
@@ -305,9 +311,6 @@ static void _add_hiscore_row(MenuScroller* scroller, scorefile_entry& se, int id
     TextItem* tmp = nullptr;
     tmp = new TextItem();
 
-    coord_def min_coord(1,1);
-    coord_def max_coord(1,2);
-
     tmp->set_fg_colour(WHITE);
     tmp->set_highlight_colour(WHITE);
 
@@ -344,12 +347,9 @@ static void _construct_hiscore_table(MenuScroller* scroller)
 
 static void _show_morgue(scorefile_entry& se)
 {
-    formatted_scroller morgue_file;
-    int flags = MF_NOSELECT | MF_ALWAYS_SHOW_MORE | MF_NOWRAP;
-    if (Options.easy_exit_menu)
-        flags |= MF_EASY_EXIT;
+    int flags = FS_PREWRAPPED_TEXT;
+    formatted_scroller morgue_file(flags);
 
-    morgue_file.set_flags(flags, false);
     morgue_file.set_tag("morgue");
     morgue_file.set_more();
 
@@ -358,7 +358,7 @@ static void _show_morgue(scorefile_entry& se)
                          + strip_filename_unsafe_chars(morgue_base) + ".txt";
     FILE* morgue = lk_open("r", morgue_path);
 
-    if (!morgue)
+    if (!morgue) // TODO: add an error message
         return;
 
     char buf[200];
@@ -370,13 +370,11 @@ static void _show_morgue(scorefile_entry& se)
         size_t newline_pos = line.find_last_of('\n');
         if (newline_pos != string::npos)
             line.erase(newline_pos);
-        morgue_text += "<w>" + line + "</w>" + '\n';
+        morgue_text += "<w>" + replace_all(line, "<", "<<") + "</w>" + '\n';
     }
 
     lk_close(morgue, morgue_path);
 
-    clrscr();
-
     column_composer cols(2, 40);
     cols.add_formatted(
             0,
@@ -387,29 +385,74 @@ static void _show_morgue(scorefile_entry& se)
 
     unsigned i;
     for (i = 0; i < blines.size(); ++i)
-        morgue_file.add_item_formatted_string(blines[i]);
+        morgue_file.add_formatted_string(blines[i], true);
 
-    textcolour(WHITE);
     morgue_file.show();
 }
 
-void show_hiscore_table()
+class UIHiscoresMenu : public Widget
 {
-    unwind_var<string> sprintmap(crawl_state.map, crawl_state.sprint_map);
-    const int max_line   = get_number_of_lines() - 1;
-    const int max_col    = get_number_of_cols() - 1;
+public:
+    UIHiscoresMenu() : done(false) {
+        expand_v = true;
+    };
 
-    const int scores_col_start = 4;
-    const int descriptor_col_start = 4;
+    virtual void _render() override;
+    virtual SizeReq _get_preferred_size(Direction dim, int prosp_width) override;
+    virtual void _allocate_region() override;
+    virtual bool on_event(const wm_event& event) override;
+
+    bool done;
+private:
+    PrecisionMenu menu;
+};
+
+void UIHiscoresMenu::_render()
+{
+#ifdef USE_TILE_LOCAL
+    GLW_3VF t = {(float)m_region[0], (float)m_region[1], 0}, s = {1, 1, 1};
+    glmanager->set_transform(t, s);
+#endif
+    menu.draw_menu();
+#ifdef USE_TILE_LOCAL
+    glmanager->reset_transform();
+#endif
+}
+
+SizeReq UIHiscoresMenu::_get_preferred_size(Direction dim, int prosp_width)
+{
+    SizeReq ret;
+    if (!dim)
+        ret = { 80, 100 };
+    else
+        ret = { 10, 10 };
+#ifdef USE_TILE_LOCAL
+    const FontWrapper* font = tiles.get_crt_font();
+    const int f = !dim ? font->char_width() : font->char_height();
+    ret.min *= f;
+    ret.nat *= f;
+#endif
+    return ret;
+}
+
+void UIHiscoresMenu::_allocate_region()
+{
+    menu.clear();
+
+#ifdef USE_TILE_LOCAL
+    const FontWrapper* font = tiles.get_crt_font();
+    const int max_col = m_region[2]/font->char_width();
+    const int max_line = m_region[3]/font->char_height();
+#else
+    const int max_col = m_region[2] - 1, max_line = m_region[3] - 1;
+#endif
+
+    const int scores_col_start = 1;
+    const int descriptor_col_start = 1;
     const int scores_row_start = 10;
     const int scores_col_end = max_col;
-    const int scores_row_end = max_line - 1;
+    const int scores_row_end = max_line+1;
 
-    bool smart_cursor_enabled = is_smart_cursor_enabled();
-
-    clrscr();
-
-    PrecisionMenu menu;
     menu.set_select_type(PrecisionMenu::PRECISION_SINGLESELECT);
 
     MenuScroller* score_entries = new MenuScroller();
@@ -421,7 +464,7 @@ void show_hiscore_table()
 
     MenuDescriptor* descriptor = new MenuDescriptor(&menu);
     descriptor->init(coord_def(descriptor_col_start, 1),
-            coord_def(get_number_of_cols(), scores_row_start - 1),
+            coord_def(max_col+1, scores_row_start - 1),
             "descriptor");
 
 #ifdef USE_TILE_LOCAL
@@ -437,19 +480,6 @@ void show_hiscore_table()
     freeform->allow_focus(false);
     freeform->set_visible(true);
 
-    NoSelectTextItem* tmp = new NoSelectTextItem();
-    string text = "[  Up/Down or PgUp/PgDn to scroll.         Esc or R-click "
-        "exits.  ]";
-    tmp->set_text(text);
-    tmp->set_bounds(coord_def(1, max_line - 1), coord_def(max_col - 1, max_line));
-    tmp->set_fg_colour(CYAN);
-    freeform->attach_item(tmp);
-    tmp->set_visible(true);
-
-#ifdef USE_TILE_LOCAL
-    tiles.get_crt()->attach_menu(&menu);
-#endif
-
     score_entries->set_visible(true);
     descriptor->set_visible(true);
     highlighter->set_visible(true);
@@ -464,32 +494,70 @@ void show_hiscore_table()
     score_entries->activate_first_item();
 
     enable_smart_cursor(false);
-    while (true)
-    {
-        menu.draw_menu();
-        textcolour(WHITE);
-        const int keyn = getch_ck();
+}
 
-        if (keyn == CK_REDRAW)
-            continue;
-
-        if (key_is_escape(keyn) || keyn == CK_MOUSE_CMD)
-        {
-            // Go back to the menu and return the smart cursor to its previous state
-            enable_smart_cursor(smart_cursor_enabled);
-            return;
-        }
-
-        if (menu.process_key(keyn))
-        {
-            menu.clear_selections();
-            _show_morgue(*hs_list[menu.get_active_item()->get_id()]);
-            clrscr();
+bool UIHiscoresMenu::on_event(const wm_event& ev)
+{
 #ifdef USE_TILE_LOCAL
-            tiles.get_crt()->attach_menu(&menu);
-#endif
+    if (ev.type == WME_MOUSEMOTION
+     || ev.type == WME_MOUSEBUTTONDOWN
+     || ev.type == WME_MOUSEWHEEL)
+    {
+        MouseEvent mouse_ev = ev.mouse_event;
+        mouse_ev.px -= m_region[0];
+        mouse_ev.py -= m_region[1];
+
+        int key = menu.handle_mouse(mouse_ev);
+        if (key && key != CK_NO_KEY)
+        {
+            wm_event fake_key = {0};
+            fake_key.type = WME_KEYDOWN;
+            fake_key.key.keysym.sym = key;
+            on_event(fake_key);
         }
+
+        if (ev.type == WME_MOUSEMOTION)
+            _expose();
+        return true;
     }
+#endif
+
+    if (ev.type != WME_KEYDOWN)
+        return false;
+    int keyn = ev.key.keysym.sym;
+
+    if (key_is_escape(keyn) || keyn == CK_MOUSE_CMD)
+        return done = true;
+
+    if (menu.process_key(keyn))
+    {
+        menu.clear_selections();
+        _show_morgue(*hs_list[menu.get_active_item()->get_id()]);
+    }
+    _expose();
+
+    return true;
+}
+
+void show_hiscore_table()
+{
+    unwind_var<string> sprintmap(crawl_state.map, crawl_state.sprint_map);
+
+    auto vbox = make_shared<Box>(Widget::VERT);
+    auto title = make_shared<Text>(formatted_string("Dungeon Crawl Stone Soup: High Scores", YELLOW));
+    title->align_self = Widget::CENTER;
+    title->set_margin_for_sdl({0, 0, 20, 0});
+    auto hiscore_ui = make_shared<UIHiscoresMenu>();
+    vbox->add_child(move(title));
+    vbox->add_child(hiscore_ui);
+    auto popup = make_shared<ui::Popup>(move(vbox));
+
+    bool smart_cursor_enabled = is_smart_cursor_enabled();
+
+    ui::run_layout(move(popup), hiscore_ui->done);
+
+    // Go back to the menu and return the smart cursor to its previous state
+    enable_smart_cursor(smart_cursor_enabled);
 }
 
 // Trying to supply an appropriate verb for the attack type. -- bwr
@@ -757,10 +825,10 @@ actor* scorefile_entry::killer() const
 
 xlog_fields scorefile_entry::get_fields() const
 {
-    if (!fields.get())
+    if (!fields)
         return xlog_fields();
     else
-        return *fields.get();
+        return *fields;
 }
 
 bool scorefile_entry::parse(const string &line)
@@ -796,7 +864,7 @@ string scorefile_entry::raw_string() const
 
     set_score_fields();
 
-    if (!fields.get())
+    if (!fields)
         return "";
 
     return fields->xlog_line() + "\n";
@@ -1052,7 +1120,7 @@ void scorefile_entry::init_with_fields()
 
 void scorefile_entry::set_base_xlog_fields() const
 {
-    if (!fields.get())
+    if (!fields)
         fields.reset(new xlog_fields);
 
     string score_version = SCORE_VERSION;
@@ -1150,7 +1218,7 @@ void scorefile_entry::set_score_fields() const
 {
     fields.reset(new xlog_fields);
 
-    if (!fields.get())
+    if (!fields)
         return;
 
     set_base_xlog_fields();
@@ -1638,7 +1706,7 @@ void scorefile_entry::init(time_t dt)
     status_info inf;
     for (unsigned i = 0; i <= STATUS_LAST_STATUS; ++i)
     {
-        if (fill_status_info(i, &inf) && !inf.short_text.empty())
+        if (fill_status_info(i, inf) && !inf.short_text.empty())
         {
             if (!status_effects.empty())
                 status_effects += ",";
