@@ -14,7 +14,7 @@ int hiscores_new_entry(const scorefile_entry &se);
 
 void logfile_new_entry(const scorefile_entry &se);
 
-void hiscores_print_list(int display_count = -1, int format = SCORE_TERSE,
+string hiscores_print_list(int display_count = -1, int format = SCORE_TERSE,
                          int newest_entry = -1);
 void hiscores_print_all(int display_count = -1, int format = SCORE_TERSE);
 void show_hiscore_table();
