@@ -373,6 +373,18 @@ static int crawl_process_keys(lua_State *ls)
     if (!_check_can_do_command(ls))
         return 0;
 
+    // if there's pending input, pushing to the end of the buffer may separate
+    // the first element of the sequence from the rest. TODO: should this be
+    // changed to push the key sequence to the beginning of the buffer? See
+    // crawl_do_commands below.
+    if (has_pending_input())
+    {
+        luaL_error(ls,
+                "Cannot currently process new keys (there is pending input)");
+        return 0;
+    }
+
+
     const char* keys = luaL_checkstring(ls, 1);
 
     if (strlen(keys) == 0)
@@ -401,6 +413,16 @@ static int crawl_process_keys(lua_State *ls)
     return 0;
 }
 
+static int crawl_set_sendkeys_errors(lua_State *ls)
+{
+    // enable or disable crashing on flushing an incomplete sendkeys buffer.
+    // used for tests, not generally useful in real life. Crashing only happens
+    // in wizmode.
+    const bool errors = lua_toboolean(ls, 1);
+    crawl_state.nonempty_buffer_flush_errors = errors;
+    return 0;
+}
+
 static int crawl_do_commands(lua_State *ls)
 {
     if (!_check_can_do_command(ls))
@@ -429,10 +451,10 @@ static int crawl_do_commands(lua_State *ls)
         lua_pop(ls, 1);
     }
 
-    flush_input_buffer(FLUSH_BEFORE_COMMAND);
-
     bool first = true;
     command_type firstcmd = CMD_NO_CMD;
+    deque<command_type> cmd_seq;
+
     for (const auto& command : commands)
     {
         command_type cmd = name_to_command(command);
@@ -448,9 +470,16 @@ static int crawl_do_commands(lua_State *ls)
             first = false;
         }
         else
-            macro_sendkeys_end_add_expanded(command_to_key(cmd));
+            cmd_seq.push_front(cmd); // reverse order for adding below
     }
 
+    flush_input_buffer(FLUSH_BEFORE_COMMAND);
+
+    // insert commands to the front of the macro buffer so that they are
+    // guaranteed to be processed adjacent to firstcmd
+    for (auto c : cmd_seq)
+        macro_buf_add_cmd(c, true);
+
     process_command(firstcmd);
 
     return 0;
@@ -1137,6 +1166,7 @@ static const struct luaL_reg crawl_clib[] =
     { "sendkeys",           crawl_sendkeys },
     { "process_command",    crawl_process_command },
     { "process_keys",       crawl_process_keys },
+    { "set_sendkeys_errors", crawl_set_sendkeys_errors },
     { "do_commands",        crawl_do_commands },
 #ifdef USE_SOUND
     { "playsound",          crawl_playsound },
