@@ -415,9 +415,15 @@ static int l_item_do_stacks(lua_State *ls)
         const bool any_stack =
             is_stackable_item(*first)
             && any_of(begin(you.inv), end(you.inv),
-                      [&] (const item_def &item) -> bool
+                      [first] (const item_def &item) -> bool
                       {
                           return items_stack(*first, item);
+                      })
+         || first->base_type == OBJ_WANDS
+            && any_of(begin(you.inv), end(you.inv),
+                      [first] (const item_def &item) -> bool
+                      {
+                          return item.is_type(OBJ_WANDS, first->sub_type);
                       });
         lua_pushboolean(ls, any_stack);
     }
