@@ -123,6 +123,7 @@ LUARET1(you_res_fire, number, player_res_fire(false))
 LUARET1(you_res_cold, number, player_res_cold(false))
 LUARET1(you_res_draining, number, player_prot_life(false))
 LUARET1(you_res_shock, number, player_res_electricity(false))
+LUARET1(you_res_drowning, boolean, you.res_water_drowning())
 LUARET1(you_res_mutation, number, you.rmut_from_item(false) ? 1 : 0)
 LUARET1(you_see_invisible, boolean, you.can_see_invisible(false))
 // Returning a number so as not to break existing scripts.
@@ -190,6 +191,7 @@ LUARET1(you_has_claws, number, you.has_claws(false))
 LUARET1(you_temp_mutations, number, you.attribute[ATTR_TEMP_MUTATIONS])
 LUARET1(you_mutation_overview, string, mutation_overview().c_str())
 
+LUARET1(you_los, number, get_los_radius())
 LUARET1(you_see_cell_rel, boolean,
         you.see_cell(coord_def(luaL_checkint(ls, 1), luaL_checkint(ls, 2)) + you.pos()))
 LUARET1(you_see_cell_no_trans_rel, boolean,
@@ -564,7 +566,7 @@ LUAFN(you_status)
     status_info inf;
     for (unsigned i = 0; i <= STATUS_LAST_STATUS; ++i)
     {
-        if (fill_status_info(i, &inf) && !inf.short_text.empty())
+        if (fill_status_info(i, inf) && !inf.short_text.empty())
         {
             if (which)
             {
@@ -633,6 +635,7 @@ static const struct luaL_reg you_clib[] =
     { "res_cold"    , you_res_cold   },
     { "res_draining", you_res_draining },
     { "res_shock"   , you_res_shock },
+    { "res_drowning", you_res_drowning },
     { "res_mutation", you_res_mutation },
     { "see_invisible", you_see_invisible },
     { "spirit_shield", you_spirit_shield },
@@ -700,6 +703,7 @@ static const struct luaL_reg you_clib[] =
     { "can_smell",         you_can_smell },
     { "has_claws",         you_has_claws },
 
+    { "los",               you_los },
     { "see_cell",          you_see_cell_rel },
     { "see_cell_no_trans", you_see_cell_no_trans_rel },
     { "see_cell_solid",    you_see_cell_solid_rel },
