@@ -15,6 +15,7 @@
 #include "colour.h"
 #include "cloud.h"
 #include "database.h"
+#include "dbg-util.h"
 #include "decks.h"
 #include "describe.h"
 #include "describe-god.h"
@@ -68,10 +69,8 @@ enum class lookup_type
     db_suffix       = 1<<0,
     /// whether the sorting functionality should be turned off
     disable_sort    = 1<<1,
-    /// whether the display menu for this supports tiles
-    support_tiles   = 1<<2,
     /// whether the display menu for this has toggleable sorting
-    toggleable_sort = 1<<3,
+    toggleable_sort = 1<<2,
 };
 DEF_BITFIELD(lookup_type_flags, lookup_type);
 
@@ -142,7 +141,8 @@ private:
     /// a function taking a single character & returning a list of keys
     /// corresponding to that glyph
     keys_by_glyph glyph_fetch;
-    /// no idea, sorry :(
+    /// take the list of keys that were automatically found and fix them
+    /// up if necessary
     db_keys_recap recap;
     /// take a letter & a key, return a corresponding new menu entry
     menu_entry_generator menu_gen;
@@ -159,21 +159,28 @@ private:
  * @param soh_name  The name of the monster; e.g. "the Serpent of Hell dis".
  * @return          The corresponding enum; e.g. MONS_SERPENT_OF_HELL_DIS.
  */
-static monster_type _soh_type(string soh_name)
+static monster_type _soh_type(string &soh_name)
 {
-    // trying to minimize code duplication...
-    static const vector<monster_type> soh_types = {
-        MONS_SERPENT_OF_HELL, MONS_SERPENT_OF_HELL_DIS,
-        MONS_SERPENT_OF_HELL_COCYTUS, MONS_SERPENT_OF_HELL_TARTARUS,
-    };
+    const string flavour = lowercase_string(soh_name.substr(soh_name.find_last_of(' ')+1));
 
-    // grab 'cocytus' etc
-    const string flavour_name
-        = soh_name.substr(lowercase(soh_name).find("hell") + 5);
-    for (monster_type mtype : soh_types)
-        if (serpent_of_hell_flavour(mtype) == flavour_name)
-            return mtype;
-    return MONS_PROGRAM_BUG;
+    branch_type branch = NUM_BRANCHES;
+    for (int b = BRANCH_FIRST_HELL; b <= BRANCH_LAST_HELL; ++b)
+        if (ends_with(flavour, lowercase_string(branches[b].shortname)))
+            branch = (branch_type)b;
+
+    switch (branch)
+    {
+        case BRANCH_COCYTUS:
+            return MONS_SERPENT_OF_HELL_COCYTUS;
+        case BRANCH_DIS:
+            return MONS_SERPENT_OF_HELL_DIS;
+        case BRANCH_TARTARUS:
+            return MONS_SERPENT_OF_HELL_TARTARUS;
+        case BRANCH_GEHENNA:
+            return MONS_SERPENT_OF_HELL;
+        default:
+            die("bad serpent of hell name");
+    }
 }
 
 static bool _is_soh(string name)
@@ -181,6 +188,12 @@ static bool _is_soh(string name)
     return starts_with(lowercase(name), "the serpent of hell");
 }
 
+static string _soh_name(monster_type m_type)
+{
+    branch_type b = serpent_of_hell_branch(m_type);
+    return string("The Serpent of Hell (") + branches[b].longname + ")";
+}
+
 static monster_type _mon_by_name(string name)
 {
     return _is_soh(name) ? _soh_type(name) : get_monster_by_name(name);
@@ -224,8 +237,7 @@ static bool _compare_mon_toughness(MenuEntry *entry_a, MenuEntry* entry_b)
 class DescMenu : public Menu
 {
 public:
-    DescMenu(int _flags, bool _toggleable_sort, bool _text_only)
-    : Menu(_flags, "", _text_only), sort_alpha(true),
+    DescMenu(int _flags, bool _toggleable_sort) : Menu(_flags, ""), sort_alpha(true),
     toggleable_sort(_toggleable_sort)
     {
         set_highlighter(nullptr);
@@ -265,7 +277,7 @@ public:
 
         for (unsigned int i = 0, size = items.size(); i < size; i++)
         {
-            const char letter = index_to_letter(i);
+            const char letter = index_to_letter(i % 52);
 
             items[i]->hotkeys.clear();
             items[i]->add_hotkey(letter);
@@ -413,7 +425,7 @@ static bool _spell_filter(string key, string body)
 
     if (spell == SPELL_NO_SPELL)
         return true;
-    if (spell_removed(spell))
+    if (spell_removed(spell) && spell != SPELL_ABJURATION)
         return true;
 
     if (get_spell_flags(spell) & SPFLAG_TESTING)
@@ -541,45 +553,6 @@ static void _recap_card_keys(vector<string> &keys)
     }
 }
 
-/**
- * Make a list of all books that contain a given spell.
- *
- * @param spell_type spell      The spell in question.
- * @return                      A formatted list of books containing
- *                              the spell, e.g.:
- *    \n\nThis spell can be found in the following books: dreams, burglary.
- *    or
- *    \n\nThis spell is not found in any books.
- */
-static string _spell_sources(const spell_type spell)
-{
-    item_def item;
-    set_ident_flags(item, ISFLAG_IDENT_MASK);
-    vector<string> books;
-
-    item.base_type = OBJ_BOOKS;
-    for (int i = 0; i < NUM_FIXED_BOOKS; i++)
-        for (spell_type sp : spellbook_template(static_cast<book_type>(i)))
-            if (sp == spell)
-            {
-                item.sub_type = i;
-                books.push_back(item.name(DESC_PLAIN));
-            }
-
-    if (books.empty())
-        return "\n\nThis spell is not found in any books.";
-
-    string desc;
-
-    desc += "\n\nThis spell can be found in the following book";
-    if (books.size() > 1)
-        desc += "s";
-    desc += ":\n ";
-    desc += comma_separated_line(books.begin(), books.end(), "\n ", "\n ");
-
-    return desc;
-}
-
 /**
  * Make a basic, no-frills ?/<foo> menu entry.
  *
@@ -609,13 +582,13 @@ static MenuEntry* _monster_menu_gen(char letter, const string &str,
     // Create and store fake monsters, so the menu code will
     // have something valid to refer to.
     monster_type m_type = _mon_by_name(str);
-    const string name = _is_soh(str) ? "The Serpent of Hell" : str;
+    const string name = _is_soh(str) ? _soh_name(m_type) : str;
 
     monster_type base_type = MONS_NO_MONSTER;
     // HACK: Set an arbitrary humanoid monster as base type.
     if (mons_class_is_zombified(m_type))
         base_type = MONS_GOBLIN;
-    // FIXME: This doesn't generate proper draconian monsters.
+
     monster_info fake_mon(m_type, base_type);
     fake_mon.props["fake"] = true;
 
@@ -670,16 +643,15 @@ static MenuEntry* _item_menu_gen(char letter, const string &str, string &key)
  */
 static MenuEntry* _feature_menu_gen(char letter, const string &str, string &key)
 {
-    const dungeon_feature_type feat = feat_by_desc(str);
     MenuEntry* me = new MenuEntry(str, MEL_ITEM, 1, letter);
     me->data = &key;
 
 #ifdef USE_TILE
+    const dungeon_feature_type feat = feat_by_desc(str);
     if (feat)
     {
         const tileidx_t idx = tileidx_feature_base(feat);
-        me->add_tile(tile_def(pick_dngn_tile(idx, ui_random(INT_MAX)),
-                                             get_dngn_tex(idx)));
+        me->add_tile(tile_def(idx, get_dngn_tex(idx)));
     }
 #endif
 
@@ -762,8 +734,10 @@ static MenuEntry* _branch_menu_gen(char letter, const string &str, string &key)
 {
     MenuEntry* me = _simple_menu_gen(letter, str, key);
 
-#ifdef USE_TILE
     const branch_type branch = branch_by_shortname(str);
+    int hotkey = branches[branch].travel_shortcut;
+    me->hotkeys = {hotkey, tolower(hotkey)};
+#ifdef USE_TILE
     me->add_tile(tile_def(tileidx_branch(branch), TEX_FEAT));
 #endif
 
@@ -863,24 +837,16 @@ static string _mons_desc_key(monster_type type)
  */
 void LookupType::display_keys(vector<string> &key_list) const
 {
-    // For tiles builds use a tiles menu to display monsters.
-    const bool text_only =
-#ifdef USE_TILE_LOCAL
-    !(flags & lookup_type::support_tiles);
-#else
-    true;
-#endif
-
-    DescMenu desc_menu(MF_SINGLESELECT | MF_ANYPRINTABLE | MF_ALLOW_FORMATTING,
-                       toggleable_sort(), text_only);
+    DescMenu desc_menu(MF_SINGLESELECT | MF_ANYPRINTABLE | MF_ALLOW_FORMATTING
+            | MF_NO_SELECT_QTY | MF_USE_TWO_COLUMNS , toggleable_sort());
     desc_menu.set_tag("description");
 
     // XXX: ugh
     const bool doing_mons = type == "monster";
-    monster_info monster_list[52];
+    vector<monster_info> monster_list(key_list.size());
     for (unsigned int i = 0, size = key_list.size(); i < size; i++)
     {
-        const char letter = index_to_letter(i);
+        const char letter = index_to_letter(i % 52);
         string &key = key_list[i];
         // XXX: double ugh
         if (doing_mons)
@@ -894,34 +860,31 @@ void LookupType::display_keys(vector<string> &key_list) const
 
     desc_menu.sort();
 
-    while (true)
+    desc_menu.on_single_selection = [this, doing_mons](const MenuEntry& item)
     {
-        vector<MenuEntry*> sel = desc_menu.show();
-        redraw_screen();
-        if (sel.empty())
+        ASSERT(item.hotkeys.size() >= 1);
+
+        string key;
+
+        if (doing_mons)
         {
-            if (toggleable_sort() && desc_menu.getkey() == CONTROL('S'))
-                desc_menu.toggle_sorting();
-            else
-                return; // only exit from this function
+            monster_info* mon = (monster_info*) item.data;
+            key = _mons_desc_key(mon->type);
         }
         else
-        {
-            ASSERT(sel.size() == 1);
-            ASSERT(sel[0]->hotkeys.size() >= 1);
+            key = *((string*) item.data);
 
-            string key;
+        describe(key);
+        return true;
+    };
 
-            if (doing_mons)
-            {
-                monster_info* mon = (monster_info*) sel[0]->data;
-                key = _mons_desc_key(mon->type);
-            }
-            else
-                key = *((string*) sel[0]->data);
-
-            describe(key);
-        }
+    while (true)
+    {
+        desc_menu.show();
+        if (toggleable_sort() && desc_menu.getkey() == CONTROL('S'))
+            desc_menu.toggle_sorting();
+        else
+            break;
     }
 }
 
@@ -975,7 +938,8 @@ int LookupType::describe(const string &key, bool exact_match) const
  * @return              The keypress the user made to exit.
  */
 static int _describe_key(const string &key, const string &suffix,
-                         string footer, const string &extra_info)
+                         string footer, const string &extra_info,
+                         const tile_def *tile = nullptr)
 {
     describe_info inf;
     inf.quote = getQuoteString(key);
@@ -993,12 +957,7 @@ static int _describe_key(const string &key, const string &suffix,
     inf.footer = footer;
     inf.title  = title;
 
-#ifdef USE_TILE_WEB
-    tiles_crt_control show_as_menu(CRT_MENU, "description");
-#endif
-
-    print_description(inf);
-    return getchm();
+    return show_description(inf, tile);
 }
 
 /**
@@ -1035,7 +994,14 @@ static int _describe_monster(const string &key, const string &suffix,
     if (mons_is_ghost_demon(mon_num) || mons_class_is_zombified(mon_num))
         return _describe_generic(key, suffix, footer);
 
-    monster_info mi(mon_num);
+    monster_type base_type = MONS_NO_MONSTER;
+    // Might be better to show all possible combinations rather than picking
+    // one at random as this does?
+    if (mons_is_draconian_job(mon_num))
+        base_type = random_draconian_monster_species();
+    else if (mons_is_demonspawn_job(mon_num))
+        base_type = random_demonspawn_monster_species();
+    monster_info mi(mon_num, base_type);
     // Avoid slime creature being described as "buggy"
     if (mi.type == MONS_SLIME_CREATURE)
         mi.slime_size = 1;
@@ -1057,12 +1023,27 @@ static int _describe_spell(const string &key, const string &suffix,
     const string spell_name = key.substr(0, key.size() - suffix.size());
     const spell_type spell = spell_by_name(spell_name, true);
     ASSERT(spell != SPELL_NO_SPELL);
-
-    const string spell_info = player_spell_desc(spell);
-    const string source_info = _spell_sources(spell);
-    return _describe_key(key, suffix, footer, spell_info + source_info);
+    describe_spell(spell, nullptr, nullptr, true);
+    return 0;
 }
 
+static int _describe_skill(const string &key, const string &suffix,
+                             string footer)
+{
+    const string skill_name = key.substr(0, key.size() - suffix.size());
+    const skill_type skill = skill_from_name(skill_name.c_str());
+    describe_skill(skill);
+    return 0;
+}
+
+static int _describe_ability(const string &key, const string &suffix,
+                             string footer)
+{
+    const string abil_name = key.substr(0, key.size() - suffix.size());
+    const ability_type abil = ability_by_name(abil_name.c_str());
+    describe_ability(abil);
+    return 0;
+}
 
 /**
  * Describe the card with the given name.
@@ -1078,7 +1059,12 @@ static int _describe_card(const string &key, const string &suffix,
     const string card_name = key.substr(0, key.size() - suffix.size());
     const card_type card = name_to_card(card_name);
     ASSERT(card != NUM_CARDS);
+#ifdef USE_TILE
+    tile_def tile = tile_def(TILE_MISC_CARD, TEX_DEFAULT);
+    return _describe_key(key, suffix, footer, which_decks(card) + "\n", &tile);
+#else
     return _describe_key(key, suffix, footer, which_decks(card) + "\n");
+#endif
 }
 
 /**
@@ -1095,29 +1081,17 @@ static int _describe_cloud(const string &key, const string &suffix,
     const string cloud_name = key.substr(0, key.size() - suffix.size());
     const cloud_type cloud = cloud_name_to_type(cloud_name);
     ASSERT(cloud != NUM_CLOUD_TYPES);
+#ifdef USE_TILE
+    cloud_info fake_cloud_info;
+    fake_cloud_info.type = cloud;
+    const tileidx_t idx = tileidx_cloud(fake_cloud_info) & ~TILE_FLAG_FLYING;
+    tile_def tile = tile_def(idx, TEX_DEFAULT);
+    return _describe_key(key, suffix, footer, extra_cloud_info(cloud), &tile);
+#else
     return _describe_key(key, suffix, footer, extra_cloud_info(cloud));
+#endif
 }
 
-
-/**
- * Describe the given spellbook.
- *
- * @param item      The item in question.
- * @return          0.
- *                  TODO: change to the last keypress (to allow exact match
- *                  support)
- */
-static int _describe_spellbook(const item_def &item)
-{
-    const string desc = get_item_description(item, true);
-    formatted_string fdesc;
-    fdesc.cprintf("%s", desc.c_str());
-
-    list_spellset(item_spellset(item), nullptr, &item, fdesc);
-    return 0; // XXX: this breaks exact match stuff
-}
-
-
 /**
  * Describe the item with the given name.
  *
@@ -1130,23 +1104,19 @@ static int _describe_item(const string &key, const string &suffix,
                            string footer)
 {
     item_def item;
-    string stats;
-    if (get_item_by_name(&item, key.c_str(), OBJ_WEAPONS)
-        || get_item_by_name(&item, key.c_str(), OBJ_ARMOUR)
-        || get_item_by_name(&item, key.c_str(), OBJ_MISSILES)
-        || get_item_by_name(&item, key.c_str(), OBJ_MISCELLANY))
-    {
-        // don't request description since _describe_key handles that
-        stats = get_item_description(item, true, false, true);
-    }
-    // spellbooks are interactive & so require special handling
-    else if (get_item_by_name(&item, key.c_str(), OBJ_BOOKS))
-    {
-        item_colour(item);
-        return _describe_spellbook(item);
-    }
+    if (!get_item_by_exact_name(item, key.c_str()))
+        die("Unable to get item %s by name", key.c_str());
+    describe_item(item);
+    return 0;
+}
 
-    return _describe_key(key, suffix, footer, stats);
+static int _describe_feature(const string &key, const string &suffix,
+                             string footer)
+{
+    const string feat_name = key.substr(0, key.size() - suffix.size());
+    const dungeon_feature_type feat = feat_by_desc(feat_name);
+    describe_feature_type(feat);
+    return 0;
 }
 
 /**
@@ -1160,11 +1130,7 @@ static int _describe_god(const string &key, const string &/*suffix*/,
 {
     const god_type which_god = str_to_god(key);
     ASSERT(which_god != GOD_NO_GOD);
-
-#ifdef USE_TILE_WEB
-    tiles_crt_control show_as_menu(CRT_MENU, "describe_god");
-#endif
-    describe_god(which_god, true);
+    describe_god(which_god);
 
     return 0; // no exact matches for gods, so output doesn't matter
 }
@@ -1277,55 +1243,49 @@ static int _describe_branch(const string &key, const string &suffix,
             + "\n\n"
             + branch_rune_desc(branch, false);
 
+#ifdef USE_TILE
+    tile_def tile = tile_def(tileidx_branch(branch), TEX_FEAT);
+    return _describe_key(key, suffix, footer, info, &tile);
+#else
     return _describe_key(key, suffix, footer, info);
+#endif
 }
 
 /// All types of ?/ queries the player can enter.
 static const vector<LookupType> lookup_types = {
     LookupType('M', "monster", _recap_mon_keys, _monster_filter,
                _get_monster_keys, nullptr, nullptr,
-               _describe_monster,
-               lookup_type::support_tiles | lookup_type::toggleable_sort),
+               _describe_monster, lookup_type::toggleable_sort),
     LookupType('S', "spell", _recap_spell_keys, _spell_filter,
                nullptr, nullptr, _spell_menu_gen,
-               _describe_spell,
-               lookup_type::db_suffix | lookup_type::support_tiles),
+               _describe_spell, lookup_type::db_suffix),
     LookupType('K', "skill", nullptr, nullptr,
                nullptr, _get_skill_keys, _skill_menu_gen,
-               _describe_generic,
-               lookup_type::support_tiles),
+               _describe_skill, lookup_type::none),
     LookupType('A', "ability", _recap_ability_keys, _ability_filter,
                nullptr, nullptr, _ability_menu_gen,
-               _describe_generic,
-               lookup_type::db_suffix | lookup_type::support_tiles),
+               _describe_ability, lookup_type::db_suffix),
     LookupType('C', "card", _recap_card_keys, _card_filter,
                nullptr, nullptr, _card_menu_gen,
-               _describe_card,
-               lookup_type::db_suffix | lookup_type::support_tiles),
+               _describe_card, lookup_type::db_suffix),
     LookupType('I', "item", nullptr, _item_filter,
                item_name_list_for_glyph, nullptr, _item_menu_gen,
-               _describe_item,
-               lookup_type::none | lookup_type::support_tiles),
+               _describe_item, lookup_type::none),
     LookupType('F', "feature", _recap_feat_keys, _feature_filter,
                nullptr, nullptr, _feature_menu_gen,
-               _describe_generic,
-               lookup_type::support_tiles),
+               _describe_feature, lookup_type::none),
     LookupType('G', "god", nullptr, nullptr,
                nullptr, _get_god_keys, _god_menu_gen,
-               _describe_god,
-               lookup_type::support_tiles),
+               _describe_god, lookup_type::none),
     LookupType('B', "branch", nullptr, nullptr,
                nullptr, _get_branch_keys, _branch_menu_gen,
-               _describe_branch,
-               lookup_type::disable_sort | lookup_type::support_tiles),
+               _describe_branch, lookup_type::disable_sort),
     LookupType('L', "cloud", nullptr, nullptr,
                nullptr, _get_cloud_keys, _cloud_menu_gen,
-               _describe_cloud,
-               lookup_type::db_suffix | lookup_type::support_tiles),
+               _describe_cloud, lookup_type::db_suffix),
     LookupType('T', "status", nullptr, _status_filter,
                nullptr, nullptr, _simple_menu_gen,
-               _describe_generic,
-               lookup_type::db_suffix),
+               _describe_generic, lookup_type::db_suffix),
 };
 
 /**
@@ -1412,18 +1372,6 @@ static string _keylist_invalid_reason(const vector<string> &key_list,
         return "No matching " + plur_type + ".";
     }
 
-    if (key_list.size() > 52)
-    {
-        if (by_symbol)
-        {
-            return "Too many " + plur_type + " with symbol '" + regex +
-                    "' to display.";
-        }
-
-        return make_stringf("Too many matching %s (%d) to display.",
-                            plur_type.c_str(), (int) key_list.size());
-    }
-
     // we're good!
     return "";
 }
