@@ -217,7 +217,7 @@ enum mon_resist_flags
     // unused 1 << 25,
 #endif
     MR_RES_STICKY_FLAME  = 1 << 26,
-    MR_RES_WIND          = 1 << 27,
+    MR_RES_TORNADO       = 1 << 27,
     MR_RES_STEAM         = 1 << 28,
 
     // vulnerabilities
