@@ -638,7 +638,7 @@ monster_info::monster_info(const monster* m, int milev)
 
     if (mons_is_pghost(type))
     {
-        ASSERT(m->ghost.get());
+        ASSERT(m->ghost);
         ghost_demon& ghost = *m->ghost;
         i_ghost.species = ghost.species;
         if (species_is_draconian(i_ghost.species) && ghost.xl < 7)
@@ -651,6 +651,8 @@ monster_info::monster_info(const monster* m, int milev)
         i_ghost.ac = quantise(ghost.ac, 5);
         i_ghost.damage = ghost.damage;
         props[KNOWN_MAX_HP_KEY] = (int)ghost.max_hp;
+        if (m->props.exists(MIRRORED_GHOST_KEY))
+            props[MIRRORED_GHOST_KEY] = m->props[MIRRORED_GHOST_KEY];
 
         // describe abnormal (branded) ghost weapons
         if (ghost.brand != SPWPN_NORMAL)
@@ -818,7 +820,7 @@ static string _mutant_beast_facet(int facet)
 
 string monster_info::db_name() const
 {
-    if (type == MONS_DANCING_WEAPON && inv[MSLOT_WEAPON].get())
+    if (type == MONS_DANCING_WEAPON && inv[MSLOT_WEAPON])
     {
         iflags_t ignore_flags = ISFLAG_KNOW_CURSE | ISFLAG_KNOW_PLUSES;
         bool     use_inscrip  = false;
@@ -886,7 +888,7 @@ string monster_info::_core_name() const
 
         case MONS_DANCING_WEAPON:
         case MONS_SPECTRAL_WEAPON:
-            if (inv[MSLOT_WEAPON].get())
+            if (inv[MSLOT_WEAPON])
             {
                 iflags_t ignore_flags = ISFLAG_KNOW_CURSE | ISFLAG_KNOW_PLUSES;
                 bool     use_inscrip  = true;
@@ -1099,10 +1101,15 @@ bool monster_info::less_than_wrapper(const monster_info& m1,
 bool monster_info::less_than(const monster_info& m1, const monster_info& m2,
                              bool zombified, bool fullname)
 {
-    if (mons_is_hepliaklqana_ancestor(m1.type))
-        return true;
-    else if (mons_is_hepliaklqana_ancestor(m2.type))
+    // This awkward ordering (checking m2 before m1) is required to satisfy
+    // std::sort's contract. Specifically, if m1 and m2 are both ancestors
+    // (e.g. through phantom mirror), we want them to compare equal. To signify
+    // "equal", we must say that neither is less than the other, rather than
+    // saying that both are less than each other.
+    if (mons_is_hepliaklqana_ancestor(m2.type))
         return false;
+    else if (mons_is_hepliaklqana_ancestor(m1.type))
+        return true;
 
     if (m1.attitude < m2.attitude)
         return true;
