@@ -32,6 +32,8 @@ const int KRAKEN_TENTACLE_RANGE = 3;
 
 #define DROPPER_MID_KEY "dropper_mid"
 
+#define MAP_KEY "map"
+
 typedef map<enchant_type, mon_enchant> mon_enchant_list;
 
 struct monsterentry;
@@ -382,7 +384,7 @@ public:
     int res_negative_energy(bool intrinsic_only = false) const override;
     bool res_torment() const override;
     int res_acid(bool calc_unid = true) const override;
-    bool res_wind() const override;
+    bool res_tornado() const override;
     bool res_petrify(bool /*temp*/ = true) const override;
     int res_constrict() const override;
     int res_magic(bool calc_unid = true) const override;
