@@ -238,7 +238,7 @@ static const mutation_def mut_data[] =
   "slow metabolism",
 
   {"You have a slow metabolism.",
-   "You need consume almost no food.",
+   "You need to consume almost no food.",
    ""},
 
   {"Your metabolism slows.",
@@ -1835,7 +1835,7 @@ static const mutation_def mut_data[] =
     "Your ability to control magic is less disrupted."},
 },
 
-{ MUT_MP_WANDS, 7, 1, mutflag::bad, false,
+{ MUT_MP_WANDS, 7, 1, mutflag::good, false,
   "MP-powered wands",
 
   {"You expend magic power (3 MP) to strengthen your wands.", "", ""},
