@@ -38,6 +38,7 @@
 #include "player-equip.h" // lose_permafly_source
 #include "player-stats.h"
 #include "religion.h"
+#include "scroller.h"
 #include "skills.h"
 #include "state.h"
 #include "stringutil.h"
@@ -46,6 +47,8 @@
 #include "viewchar.h"
 #include "xom.h"
 
+using namespace ui;
+
 static bool _delete_single_mutation_level(mutation_type mutat, const string &reason, bool transient);
 
 struct body_facet_def
@@ -372,15 +375,13 @@ mutation_activity_type mutation_activity_level(mutation_type mut)
     {
         return mutation_activity_type::INACTIVE;
     }
-
-    if (you_worship(GOD_DITHMENOS) && mut == MUT_IGNITE_BLOOD)
-        return mutation_activity_type::INACTIVE;
-
+#if TAG_MAJOR_VERSION == 34
     if ((you_worship(GOD_PAKELLAS) || player_under_penance(GOD_PAKELLAS))
          && (mut == MUT_MANA_LINK || mut == MUT_MANA_REGENERATION))
     {
         return mutation_activity_type::INACTIVE;
     }
+#endif
 
     if (!form_can_bleed(you.form) && mut == MUT_SANGUINE_ARMOUR)
         return mutation_activity_type::INACTIVE;
@@ -589,29 +590,23 @@ void validate_mutations(bool debug_msg)
         ASSERT(you.attribute[ATTR_TEMP_MUT_XP] > 0);
 }
 
-string describe_mutations(bool center_title)
+string describe_mutations(bool drop_title)
 {
 #ifdef DEBUG
     validate_mutations(true);
 #endif
     string result;
-    const char *mut_title = "Innate Abilities, Weirdness & Mutations";
 
     _num_full_suppressed = _num_part_suppressed = 0;
     _num_transient = 0;
 
-    if (center_title)
+    if (!drop_title)
     {
-        int offset = 39 - strwidth(mut_title) / 2;
-        if (offset < 0) offset = 0;
-
-        result += string(offset, ' ');
+        result += "<white>";
+        result += "Innate Abilities, Weirdness & Mutations";
+        result += "</white>\n\n";
     }
 
-    result += "<white>";
-    result += mut_title;
-    result += "</white>\n\n";
-
     result += "<lightblue>";
     const string old_result = result;
 
@@ -759,16 +754,40 @@ string describe_mutations(bool center_title)
     return result;
 }
 
-static const string _vampire_Ascreen_footer = (
+static formatted_string _vampire_Ascreen_footer(bool first_page)
+{
+    const char *text = first_page ? "<w>Mutations</w>|Blood properties"
+                                  : "Mutations|<w>Blood properties</w>";
+    const string fmt = make_stringf("[<w>!</w>/<w>^</w>"
 #ifdef USE_TILE_LOCAL
-    "<w>Right-click</w> or press '<w>!</w>'"
-#else
-    "Press '<w>!</w>'"
+            "|<w>Right-click</w>"
 #endif
-    " to toggle between mutations and properties depending on your blood\n"
-    "level.\n");
+            "]: %s", text);
+    return formatted_string::parse_string(fmt);
+}
 
-static void _display_vampire_attributes()
+static int _vampire_bloodlessness()
+{
+    switch (you.hunger_state)
+    {
+    case HS_ENGORGED:
+    case HS_VERY_FULL:
+    case HS_FULL:
+        return 1;
+    case HS_SATIATED:
+        return 2;
+    case HS_HUNGRY:
+    case HS_VERY_HUNGRY:
+    case HS_NEAR_STARVING:
+        return 3;
+    case HS_STARVING:
+    case HS_FAINTING:
+        return 4;
+    }
+    die("bad hunger state %d", you.hunger_state);
+}
+
+static string _display_vampire_attributes()
 {
     ASSERT(you.species == SP_VAMPIRE);
 
@@ -805,26 +824,7 @@ static void _display_vampire_attributes()
          "berserk              ", "yes        ", "yes        ", "no         ", "no    "}
     };
 
-    int current = 0;
-    switch (you.hunger_state)
-    {
-    case HS_ENGORGED:
-    case HS_VERY_FULL:
-    case HS_FULL:
-        current = 1;
-        break;
-    case HS_SATIATED:
-        current = 2;
-        break;
-    case HS_HUNGRY:
-    case HS_VERY_HUNGRY:
-    case HS_NEAR_STARVING:
-        current = 3;
-        break;
-    case HS_STARVING:
-    case HS_FAINTING:
-        current = 4;
-    }
+    int current = _vampire_bloodlessness();
 
     for (int y = 0; y < lines; y++)  // lines   (properties)
     {
@@ -839,18 +839,8 @@ static void _display_vampire_attributes()
         result += "\n";
     }
 
-    result += "\n";
-    result += _vampire_Ascreen_footer;
-
-    formatted_scroller attrib_menu;
-    attrib_menu.add_text(result);
-
-    attrib_menu.show();
-    if (attrib_menu.getkey() == '!'
-        || attrib_menu.getkey() == CK_MOUSE_CMD)
-    {
-        display_mutations();
-    }
+    trim_string_right(result);
+    return result;
 }
 
 void display_mutations()
@@ -864,33 +854,86 @@ void display_mutations()
         extra += "<darkgrey>(())</darkgrey>: Completely suppressed.\n";
     if (_num_transient)
         extra += "<magenta>[]</magenta>   : Transient mutations.";
-    if (you.species == SP_VAMPIRE)
-    {
-        if (!extra.empty())
-            extra += "\n";
-
-        extra += _vampire_Ascreen_footer;
-    }
 
     if (!extra.empty())
     {
         mutation_s += "\n\n\n\n";
         mutation_s += extra;
     }
+    trim_string_right(mutation_s);
 
-    formatted_scroller mutation_menu;
-    mutation_menu.add_text(mutation_s);
+    auto vbox = make_shared<Box>(Widget::VERT);
 
-    mouse_control mc(MOUSE_MODE_MORE);
+    const char *title_text = "Innate Abilities, Weirdness & Mutations";
+    auto title = make_shared<Text>(formatted_string(title_text, WHITE));
+    title->align_self = Widget::CENTER;
+    vbox->add_child(move(title));
 
-    mutation_menu.show();
+    auto switcher = make_shared<Switcher>();
 
-    if (you.species == SP_VAMPIRE
-        && (mutation_menu.getkey() == '!'
-            || mutation_menu.getkey() == CK_MOUSE_CMD))
+    const string vamp_s = you.species == SP_VAMPIRE ?_display_vampire_attributes() : "N/A";
+    const string descs[3] =  { mutation_s, vamp_s };
+    for (int i = 0; i < 2; i++)
     {
-        _display_vampire_attributes();
+        auto scroller = make_shared<Scroller>();
+        auto text = make_shared<Text>(formatted_string::parse_string(
+                descs[static_cast<int>(i)]));
+        text->wrap_text = true;
+        scroller->set_child(text);
+        switcher->add_child(move(scroller));
     }
+
+    switcher->current() = 0;
+    switcher->set_margin_for_sdl({20, 0, 0, 0});
+    switcher->set_margin_for_crt({1, 0, 0, 0});
+    switcher->expand_h = false;
+#ifdef USE_TILE_LOCAL
+    switcher->max_size()[0] = tiles.get_crt_font()->char_width()*80;
+#endif
+    vbox->add_child(switcher);
+
+    auto bottom = make_shared<Text>(_vampire_Ascreen_footer(true));
+    bottom->set_margin_for_sdl({20, 0, 0, 0});
+    bottom->set_margin_for_crt({1, 0, 0, 0});
+    if (you.species == SP_VAMPIRE)
+        vbox->add_child(bottom);
+
+    auto popup = make_shared<ui::Popup>(vbox);
+
+    bool done = false;
+    int lastch;
+    popup->on(Widget::slots.event, [&](wm_event ev) {
+        if (ev.type != WME_KEYDOWN)
+            return false;
+        lastch = ev.key.keysym.sym;
+        if (you.species == SP_VAMPIRE && (lastch == '!' || lastch == CK_MOUSE_CMD || lastch == '^'))
+        {
+            int c = 1 - switcher->current();
+            switcher->current() = c;
+#ifdef USE_TILE_WEB
+            tiles.json_open_object();
+            tiles.json_write_int("pane", c);
+            tiles.ui_state_change("mutations", 0);
+#endif
+            bottom->set_text(_vampire_Ascreen_footer(c));
+        } else
+            done = !vbox->on_event(ev);
+        return true;
+    });
+
+#ifdef USE_TILE_WEB
+    tiles.json_open_object();
+    tiles.json_write_string("mutations", mutation_s);
+    if (you.species == SP_VAMPIRE)
+        tiles.json_write_int("vampire", _vampire_bloodlessness());
+    tiles.push_ui_layout("mutations", 1);
+#endif
+
+    ui::run_layout(move(popup), done);
+
+#ifdef USE_TILE_WEB
+    tiles.pop_ui_layout();
+#endif
 }
 
 static int _calc_mutation_amusement_value(mutation_type which_mutation)
