@@ -185,6 +185,7 @@ public:
     int         mlist_min_height;
     int         msg_min_height;
     int         msg_max_height;
+    int         msg_webtiles_height;
     bool        mlist_allow_alternate_layout;
     bool        messages_at_top;
     bool        msg_condense_repeats;
@@ -233,6 +234,7 @@ public:
     bool        easy_eat_chunks; // make 'e' auto-eat the oldest safe chunk
     bool        auto_eat_chunks; // allow eating chunks while resting or travelling
     skill_focus_mode skill_focus; // is the focus skills available
+    bool        auto_hide_spells; // hide new spells
 
     bool        note_all_skill_levels;  // take note for all skill levels (1-27)
     bool        note_skill_max;   // take note when skills reach new max
@@ -402,7 +404,6 @@ public:
 
     int         pickup_menu_limit;  // Over this number of items, menu for
                                     // pickup
-    bool        easy_exit_menu;     // Menus are easier to get out of
     bool        ability_menu;       // 'a'bility starts with a full-screen menu
     bool        easy_floor_use;     // , selects the floor item if there's 1
 
@@ -486,29 +487,31 @@ public:
     bool        tile_menu_icons; // display icons in menus?
 
     // minimap colours
-    VColour     tile_player_col;
-    VColour     tile_monster_col;
-    VColour     tile_plant_col;
-    VColour     tile_item_col;
     VColour     tile_unseen_col;
     VColour     tile_floor_col;
     VColour     tile_wall_col;
     VColour     tile_mapped_floor_col;
     VColour     tile_mapped_wall_col;
     VColour     tile_door_col;
-    VColour     tile_downstairs_col;
+    VColour     tile_item_col;
+    VColour     tile_monster_col;
+    VColour     tile_plant_col;
     VColour     tile_upstairs_col;
+    VColour     tile_downstairs_col;
+    VColour     tile_branchstairs_col;
+    VColour     tile_feature_col;
+    VColour     tile_water_col;
+    VColour     tile_lava_col;
+    VColour     tile_trap_col;
+    VColour     tile_excl_centre_col;
+    VColour     tile_excluded_col;
+    VColour     tile_player_col;
+    VColour     tile_deep_water_col;
+    VColour     tile_portal_col;
     VColour     tile_transporter_col;
     VColour     tile_transporter_landing_col;
-    VColour     tile_branchstairs_col;
-    VColour     tile_portal_col;
-    VColour     tile_feature_col;
-    VColour     tile_trap_col;
-    VColour     tile_water_col;
-    VColour     tile_deep_water_col;
-    VColour     tile_lava_col;
-    VColour     tile_excluded_col;
-    VColour     tile_excl_centre_col;
+    VColour     tile_explore_horizon_col;
+
     VColour     tile_window_col;
 #ifdef USE_TILE_LOCAL
     // font settings
@@ -517,6 +520,7 @@ public:
     string      tile_font_stat_file;
     string      tile_font_lbl_file;
     string      tile_font_tip_file;
+    bool        tile_single_column_menus;
 #endif
 #ifdef USE_TILE_WEB
     string      tile_font_crt_family;
