@@ -271,9 +271,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
     case BEAM_AIR:
     {
         // Airstrike.
-        if (you.res_wind())
-            hurted = 0;
-        else if (you.airborne())
+        if (you.airborne())
             hurted += hurted / 2;
         break;
     }
