@@ -10,6 +10,8 @@
 const int MAX_SKILL_ORDER = 100;
 struct skill_state
 {
+    skill_state();
+
     FixedBitVector<NUM_SKILLS>            can_train;
     FixedVector<uint8_t, NUM_SKILLS>      skills;
     FixedVector<int, NUM_SKILLS>          real_skills;    // Those two are
@@ -27,8 +29,12 @@ struct skill_state
     vector<int> manual_charges;
 
     void save();
+    bool state_saved() const;
     void restore_levels();
     void restore_training();
+
+private:
+    bool saved;
 };
 
 struct skill_diff
@@ -99,7 +105,9 @@ void init_skill_order();
 bool is_removed_skill(skill_type skill);
 bool is_useless_skill(skill_type skill);
 bool is_harmful_skill(skill_type skill);
-bool all_skills_maxed(bool really_all = false);
+bool can_enable_skill(skill_type sk, bool override = false);
+bool trainable_skills(bool check_all = false);
+bool skills_being_trained();
 
 int species_apt(skill_type skill, species_type species = you.species);
 float species_apt_factor(skill_type sk, species_type sp = you.species);
@@ -122,7 +130,6 @@ int transfer_skill_points(skill_type fsk, skill_type tsk, int skp_max,
                           bool simu, bool boost = false);
 int skill_bump(skill_type skill, int scale = 1);
 void fixup_skills();
-bool can_enable_skill(skill_type sk);
 
 bool target_met(skill_type sk);
 bool check_training_target(skill_type sk);
@@ -131,8 +138,8 @@ bool check_training_targets();
 static const skill_type skill_display_order[] =
 {
     SK_TITLE,
-    SK_FIGHTING, SK_SHORT_BLADES, SK_LONG_BLADES, SK_AXES,
-    SK_MACES_FLAILS, SK_POLEARMS, SK_STAVES, SK_UNARMED_COMBAT,
+    SK_FIGHTING, SK_SHORT_BLADES, SK_LONG_BLADES,
+    SK_MACES_FLAILS, SK_AXES, SK_POLEARMS, SK_STAVES, SK_UNARMED_COMBAT,
 
     SK_BLANK_LINE,
 
