@@ -357,15 +357,6 @@ static void _remove_spell_attributes(spell_type spell)
                                  : "your spell is no longer protecting you");
         }
         break;
-#if TAG_MAJOR_VERSION == 34
-    case SPELL_DELAYED_FIREBALL:
-        if (you.attribute[ATTR_DELAYED_FIREBALL])
-        {
-            you.attribute[ATTR_DELAYED_FIREBALL] = 0;
-            mprf(MSGCH_DURATION, "Your charged fireball dissipates.");
-        }
-        break;
-#endif
     default:
         break;
     }
@@ -1205,12 +1196,7 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
             return "you must stand on solid ground to cast this.";
         }
         break;
-#if TAG_MAJOR_VERSION == 34
-    case SPELL_DELAYED_FIREBALL:
-        if (temp && you.attribute[ATTR_DELAYED_FIREBALL])
-            return "you are already charged.";
-        break;
-#endif
+
     case SPELL_BORGNJORS_REVIVIFICATION:
         if (temp && you.hp == you.hp_max)
             return "you cannot be healed further.";
@@ -1298,12 +1284,12 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
     case SPELL_POISONOUS_CLOUD:
     case SPELL_FREEZING_CLOUD:
     case SPELL_MEPHITIC_CLOUD:
-        if (env.level_state & LSTATE_STILL_WINDS)
+        if (temp && env.level_state & LSTATE_STILL_WINDS)
             return "the air is too still for clouds to form.";
         break;
 
     case SPELL_GOLUBRIAS_PASSAGE:
-        if (orb_limits_translocation())
+        if (orb_limits_translocation(temp))
             return "the Orb prevents this spell from working.";
 
     default:
