@@ -30,6 +30,7 @@
 #include "message.h"
 #include "misc.h"
 #include "mon-death.h"
+#include "movement.h"
 #include "notes.h"
 #include "orb-type.h"
 #include "output.h"
@@ -393,7 +394,7 @@ static void _rune_effect(dungeon_feature_type ftype)
         if (silenced(you.pos()))
             mpr("The gate opens wide!");
         else
-            mpr("With a loud hiss the gate opens wide!");
+            mpr("With a soft hiss the gate opens wide!");
         // these are included in default force_more_message
     }
 }
@@ -555,7 +556,8 @@ static level_id _travel_destination(const dungeon_feature_type how,
  */
 void floor_transition(dungeon_feature_type how,
                       const dungeon_feature_type whence, level_id whither,
-                      bool forced, bool going_up, bool shaft, bool update_travel_cache)
+                      bool forced, bool going_up, bool shaft,
+                      bool update_travel_cache)
 {
     const level_id old_level = level_id::current();
 
@@ -566,6 +568,15 @@ void floor_transition(dungeon_feature_type how,
     clear_trapping_net();
     end_searing_ray();
 
+    if (!forced)
+    {
+        // Break ice armour
+        remove_ice_armour_movement();
+
+        // Check for barbs and apply
+        apply_barbs_damage();
+    }
+
     // Fire level-leaving trigger.
     leaving_level_now(how);
 
