@@ -29,6 +29,7 @@
 #include "item-prop.h"
 #include "items.h"
 #include "libutil.h"
+#include "loading-screen.h"
 #include "macro.h"
 #include "maps.h"
 #include "menu.h"
@@ -55,17 +56,24 @@
 #ifdef USE_TILE
  #include "tilepick.h"
 #endif
-#ifdef USE_TILE_LOCAL
- #include "tilereg-crt.h"
-#endif
 #include "tileview.h"
 #include "viewchar.h"
 #include "view.h"
 #ifdef USE_TILE_LOCAL
  #include "windowmanager.h"
 #endif
+#include "ui.h"
 
-static void _cio_init();
+using namespace ui;
+
+static void _loading_message(string m)
+{
+    mpr(m.c_str());
+#ifdef USE_TILE_LOCAL
+    if (!crawl_state.tiles_disabled && crawl_state.title_screen)
+        loading_screen_update_msg(m.c_str());
+#endif
+}
 
 // Initialise a whole lot of stuff...
 static void _initialize()
@@ -73,6 +81,7 @@ static void _initialize()
     Options.fixup_options();
 
     you.symbol = MONS_PLAYER;
+    msg::initialise_mpr_streams();
 
     seed_rng();
 
@@ -92,7 +101,7 @@ static void _initialize()
     // be set to use with item_names_by_glyph_cache.
     init_item_name_cache();
 
-    msg::initialise_mpr_streams();
+    unwind_bool no_more(crawl_state.show_more_prompt, false);
 
     // Init item array.
     for (int i = 0; i < MAX_ITEMS; ++i)
@@ -117,28 +126,20 @@ static void _initialize()
     // may take awhile and it's better if the player can look at a pretty
     // screen while this happens.
     if (!crawl_state.tiles_disabled && crawl_state.title_screen)
-    {
-        tiles.draw_title();
-        tiles.update_title_msg("Loading databases...");
-    }
+        loading_screen_open();
 #endif
 
     // Initialise internal databases.
+    _loading_message("Loading databases...");
     databaseSystemInit();
-#ifdef USE_TILE_LOCAL
-    if (!crawl_state.tiles_disabled && crawl_state.title_screen)
-        tiles.update_title_msg("Loading spells and features...");
-#endif
 
+    _loading_message("Loading spells and features...");
     init_feat_desc_cache();
     init_spell_name_cache();
     init_spell_rarities();
-#ifdef USE_TILE_LOCAL
-    if (!crawl_state.tiles_disabled && crawl_state.title_screen)
-        tiles.update_title_msg("Loading maps...");
-#endif
 
     // Read special levels and vaults.
+    _loading_message("Loading maps...");
     read_maps();
     run_map_global_preludes();
 
@@ -146,13 +147,8 @@ static void _initialize()
         end(0);
 
 #ifdef USE_TILE_LOCAL
-    if (!crawl_state.tiles_disabled
-        && crawl_state.title_screen)
-    {
-        if (!Options.tile_skip_title)
-            tiles.update_title_msg("Loading complete, press any key to start.");
-        tiles.hide_title();
-    }
+    if (!crawl_state.tiles_disabled && crawl_state.title_screen)
+        loading_screen_close();
 #endif
 
     if (Options.seed)
@@ -176,7 +172,7 @@ static void _initialize()
     if (!crawl_state.test_list)
     {
         if (!crawl_state.io_inited)
-            _cio_init();
+            cio_init();
         clrscr();
     }
 
@@ -200,6 +196,8 @@ static void _initialize()
             "or DEBUG_TESTS)");
 #endif
     }
+
+    mpr(opening_screen().c_str());
 }
 
 /** KILL_RESETs all monsters in LOS.
@@ -247,7 +245,7 @@ static void _post_init(bool newc)
     // Load macros
     macro_init();
 
-    crawl_state.need_save = true;
+    crawl_state.need_save = crawl_state.game_started = true;
     crawl_state.last_type = crawl_state.type;
     crawl_state.marked_as_won = false;
 
@@ -308,10 +306,9 @@ static void _post_init(bool newc)
 
 #ifdef CLUA_BINDINGS
     clua.runhook("chk_startgame", "b", newc);
-    string yname = you.your_name; // XXX: what's this for?
+
     read_init_file(true);
     Options.fixup_options();
-    you.your_name = yname;
 
     // In case Lua changed the character set.
     init_char_table(Options.char_set);
@@ -570,34 +567,99 @@ static bool _game_defined(const newgame_def& ng)
 static const int SCROLLER_MARGIN_X  = 18;
 static const int NAME_START_Y       = 5;
 static const int GAME_MODES_START_Y = 7;
-static const int GAME_MODES_WIDTH   = 60;
 static const int NUM_HELP_LINES     = 3;
 static const int NUM_MISC_LINES     = 5;
 
-/**
- * Saves game mode and player name to ng_choice.
- */
-static void _show_startup_menu(newgame_def& ng_choice,
-                               const newgame_def& defaults)
-{
-    // Initialise before the loop so that ? doesn't forget the typed name.
-    string input_string = defaults.name;
+// TODO: should be game_type. Also, does this really need to be static?
+// maybe part of crawl_state?
+static int startup_menu_game_type = GAME_TYPE_UNSPECIFIED;
 
-again:
-#if defined(USE_TILE_LOCAL) && defined(TOUCH_UI)
-    wm->show_keyboard();
+class UIStartupMenu : public Widget
+{
+public:
+    UIStartupMenu(newgame_def& _ng_choice, const newgame_def &_defaults) : done(false), end_game(false), ng_choice(_ng_choice), defaults(_defaults) {
+        chars = find_all_saved_characters();
+        num_saves = chars.size();
+        input_string = crawl_state.default_startup_name;
+    };
+
+    virtual void _render() override;
+    virtual SizeReq _get_preferred_size(Direction dim, int prosp_width) override;
+    virtual void _allocate_region() override;
+    virtual bool on_event(const wm_event& event) override;
+
+    PrecisionMenu menu;
+    bool done;
+    bool end_game;
+private:
+    newgame_def& ng_choice;
+    const newgame_def &defaults;
+    string input_string;
+    bool full_name;
+    TextItem *input_text;
+    vector<player_save_info> chars;
+    int num_saves;
+    MenuScroller *game_modes, *save_games;
+    MenuDescriptor *descriptor;
+};
+
+SizeReq UIStartupMenu::_get_preferred_size(Direction dim, int prosp_width)
+{
+#ifdef USE_TILE_LOCAL
+    SizeReq ret;
+    if (!dim)
+        ret = { 80, 90 }; // seems to work well empirically
+    else
+    {
+        // duplicate of calculations below in _allocate_region()
+        const int save_lines = num_saves + (num_saves ? 1 : 0); // add 1 for New Game
+        const int help_start = GAME_MODES_START_Y + num_to_lines(save_lines + NUM_GAME_TYPE) + 2;
+        const int help_end   = help_start + NUM_HELP_LINES + 1;
+        ret = { 20, help_end };
+    }
+
+    const FontWrapper* font = tiles.get_crt_font();
+    const int f = !dim ? font->char_width() : font->char_height();
+    ret.min *= f;
+    ret.nat *= f;
+
+    return ret;
+#else
+    if (!dim)
+        return { 80, 80 };
+    else
+        return { 24, 24 };
 #endif
-    vector<player_save_info> chars = find_all_saved_characters();
-    const int num_saves = chars.size();
-    const int num_modes = NUM_GAME_TYPE;
-    static int type = GAME_TYPE_UNSPECIFIED;
+}
+
+void UIStartupMenu::_render()
+{
+#ifdef USE_TILE_LOCAL
+    GLW_3VF t = {(float)m_region[0], (float)m_region[1], 0}, s = {1, 1, 1};
+    glmanager->set_transform(t, s);
+#endif
+    ui::push_scissor(m_region);
+    menu.draw_menu();
+    ui::pop_scissor();
+#ifdef USE_TILE_LOCAL
+    glmanager->reset_transform();
+#endif
+}
+
+void UIStartupMenu::_allocate_region()
+{
+    menu.clear();
 
 #ifdef USE_TILE_LOCAL
-    const int max_col    = tiles.get_crt()->mx;
+    const FontWrapper* font = tiles.get_crt_font();
+    const int max_col = m_region[2]/font->char_width();
+    const int max_line = m_region[3]/font->char_height();
 #else
-    const int max_col    = get_number_of_cols();
+    const int max_col = m_region[2], max_line = m_region[3];
 #endif
-    const int max_line   = get_number_of_lines();
+
+    const int num_modes = NUM_GAME_TYPE;
+
     int save_lines = num_saves + (num_saves ? 1 : 0); // add 1 for New Game
     const int help_start = min(GAME_MODES_START_Y + num_to_lines(save_lines + num_modes) + 2,
                                max_line - NUM_MISC_LINES + 1);
@@ -607,19 +669,17 @@ again:
     const int game_save_top = help_start - 2 - num_to_lines(min(2, save_lines));
     const int save_games_start_y = min<int>(game_mode_bottom, game_save_top);
 
-    clrscr();
-    PrecisionMenu menu;
     menu.set_select_type(PrecisionMenu::PRECISION_SINGLESELECT);
     MenuFreeform* freeform = new MenuFreeform();
     freeform->init(coord_def(1, 1), coord_def(max_col, max_line), "freeform");
     // This freeform will only containt unfocusable texts
     freeform->allow_focus(false);
-    MenuScroller* game_modes = new MenuScroller();
+    game_modes = new MenuScroller();
     game_modes->init(coord_def(SCROLLER_MARGIN_X, GAME_MODES_START_Y),
-                     coord_def(GAME_MODES_WIDTH, save_games_start_y),
+                     coord_def(max_col, save_games_start_y),
                      "game modes");
 
-    MenuScroller* save_games = new MenuScroller();
+    save_games = new MenuScroller();
     save_games->init(coord_def(SCROLLER_MARGIN_X, save_games_start_y),
                      coord_def(max_col, help_start - 1),
                      "save games");
@@ -676,7 +736,7 @@ again:
     menu.attach_object(game_modes);
     menu.attach_object(save_games);
 
-    MenuDescriptor* descriptor = new MenuDescriptor(&menu);
+    descriptor = new MenuDescriptor(&menu);
     descriptor->init(coord_def(1, help_end), coord_def(max_col, help_end + 1),
                      "descriptor");
     menu.attach_object(descriptor);
@@ -690,10 +750,6 @@ again:
     highlighter->init(coord_def(-1, -1), coord_def(-1, -1), "highlighter");
     menu.attach_object(highlighter);
 
-#ifdef USE_TILE_LOCAL
-    tiles.get_crt()->attach_menu(&menu);
-#endif
-
     freeform->set_visible(true);
     game_modes->set_visible(true);
     save_games->set_visible(true);
@@ -701,18 +757,35 @@ again:
     highlighter->set_visible(true);
 
     // Draw legal info etc
-    opening_screen();
+    auto intro_text = new FormattedTextItem();
+    intro_text->set_text(opening_screen());
+    intro_text->set_bounds(coord_def(1, 1), coord_def(max_col, 5));
+    intro_text->set_visible(true);
+    freeform->attach_item(intro_text);
+
+    input_text = new TextItem();
+    input_text->set_text(input_string);
+    input_text->set_fg_colour(WHITE);
+    input_text->set_bounds(coord_def(SCROLLER_MARGIN_X, NAME_START_Y),
+            coord_def(max_col, NAME_START_Y+1));
+    input_text->set_visible(true);
+    freeform->attach_item(input_text);
 
     // If the game filled in a complete name, the user will
     // usually want to enter a new name instead of adding
     // to the current one.
-    bool full_name = !input_string.empty();
+    full_name = !input_string.empty();
 
     int save = _find_save(chars, input_string);
-    if (type != GAME_TYPE_UNSPECIFIED)
+    // don't use non-enum game_type values across restarts, as the list of
+    // saves may have changed on restart.
+    if (startup_menu_game_type >= NUM_GAME_TYPE)
+        startup_menu_game_type = GAME_TYPE_UNSPECIFIED;
+
+    if (startup_menu_game_type != GAME_TYPE_UNSPECIFIED)
     {
         menu.set_active_object(game_modes);
-        game_modes->set_active_item(type);
+        game_modes->set_active_item(startup_menu_game_type);
     }
     else if (save != -1)
     {
@@ -736,197 +809,263 @@ again:
         game_modes->activate_first_item();
     }
 
-    while (true)
+    descriptor->render();
+    if (recent_error_messages())
     {
-        menu.draw_menu();
-        textcolour(WHITE);
-        cgotoxy(SCROLLER_MARGIN_X, NAME_START_Y);
-        clear_to_end_of_line();
-        cgotoxy(SCROLLER_MARGIN_X, NAME_START_Y);
-        cprintf("%s", input_string.c_str());
+        descriptor->override_description(
+            "Errors during initialization; press ctrl-p to view the full log.");
+    }
+    else
+        descriptor->override_description(crawl_state.last_game_exit.message);
 
-        const int keyn = getch_ck();
+}
 
-        if (keyn == CK_REDRAW)
-            goto again;
+bool UIStartupMenu::on_event(const wm_event& ev)
+{
+#ifdef USE_TILE_LOCAL
+    if (ev.type == WME_MOUSEMOTION
+     || ev.type == WME_MOUSEBUTTONDOWN
+     || ev.type == WME_MOUSEWHEEL)
+    {
+        MouseEvent mouse_ev = ev.mouse_event;
+        mouse_ev.px -= m_region[0];
+        mouse_ev.py -= m_region[1];
 
-        if (key_is_escape(keyn) || keyn == CK_MOUSE_CMD)
+        int key = menu.handle_mouse(mouse_ev);
+        if (key && key != CK_NO_KEY)
         {
-            // End the game
-            end(0);
-        }
-        else if (keyn == '\t' && _game_defined(defaults))
-        {
-            ng_choice = defaults;
-            return;
-        }
-        else if (keyn == '?')
-        {
-            list_commands();
-
-            // If we had a save selected, reset type so that the save
-            // will continue to be selected when we restart the menu.
-            MenuItem *active = menu.get_active_item();
-            if (active && active->get_id() >= NUM_GAME_TYPE)
-                type = GAME_TYPE_UNSPECIFIED;
-
-            // restart because help messes up CRTRegion
-            goto again;
+            wm_event fake_key = {0};
+            fake_key.type = WME_KEYDOWN;
+            fake_key.key.keysym.sym = key;
+            on_event(fake_key);
         }
 
-        if (!menu.process_key(keyn))
+        if (ev.type == WME_MOUSEMOTION)
+            _expose();
+        return true;
+    }
+#endif
+
+    if (ev.type != WME_KEYDOWN)
+        return false;
+    int keyn = ev.key.keysym.sym;
+
+    // XXX: these keys are effectively broken on the main menu; there's a new
+    // menu implementation on the way, but until it's ready, disable them
+    if (keyn == CK_PGUP || keyn == CK_PGDN)
+        return true;
+
+    _expose();
+
+    if (key_is_escape(keyn) || keyn == CK_MOUSE_CMD)
+    {
+        // End the game
+        return done = end_game = true;
+    }
+    else if (keyn == '\t' && _game_defined(defaults))
+    {
+        ng_choice = defaults;
+        return done = true;
+    }
+    else if (keyn == '?')
+    {
+        show_help();
+
+        // If we had a save selected, reset type so that the save
+        // will continue to be selected when we restart the menu.
+        MenuItem *active = menu.get_active_item();
+        if (active && active->get_id() >= NUM_GAME_TYPE)
+            startup_menu_game_type = GAME_TYPE_UNSPECIFIED;
+
+        return true;
+    }
+    else if (keyn == CONTROL('P'))
+    {
+        replay_messages_during_startup();
+        MenuItem *active = menu.get_active_item();
+        if (active && active->get_id() >= NUM_GAME_TYPE)
+            startup_menu_game_type = GAME_TYPE_UNSPECIFIED;
+        return true;
+    }
+
+    if (!menu.process_key(keyn))
+    {
+        // handle the non-action keys by hand to poll input
+        // Only consider alphanumeric keys and -_ .
+        bool changed_name = false;
+        if (iswalnum(keyn) || keyn == '-' || keyn == '.'
+            || keyn == '_' || keyn == ' ')
         {
-            // handle the non-action keys by hand to poll input
-            // Only consider alphanumeric keys and -_ .
-            bool changed_name = false;
-            if (iswalnum(keyn) || keyn == '-' || keyn == '.'
-                || keyn == '_' || keyn == ' ')
+            if (full_name)
+            {
+                full_name = false;
+                input_string = "";
+            }
+            if (strwidth(input_string) < MAX_NAME_LENGTH)
+            {
+                input_string += stringize_glyph(keyn);
+                changed_name = true;
+            }
+        }
+        else if (keyn == CK_BKSP)
+        {
+            if (!input_string.empty())
             {
                 if (full_name)
-                {
-                    full_name = false;
                     input_string = "";
-                }
-                if (strwidth(input_string) < MAX_NAME_LENGTH)
-                {
-                    input_string += stringize_glyph(keyn);
-                    changed_name = true;
-                }
-            }
-            else if (keyn == CK_BKSP)
-            {
-                if (!input_string.empty())
-                {
-                    if (full_name)
-                        input_string = "";
-                    else
-                        input_string.erase(input_string.size() - 1);
-                    changed_name = true;
-                    full_name = false;
-                }
-            }
-            // clear the "That's a silly name line"
-            cgotoxy(SCROLLER_MARGIN_X, GAME_MODES_START_Y - 1);
-            clear_to_end_of_line();
-
-            // Depending on whether the current name occurs
-            // in the saved games, update the active object.
-            // We want enter to start a new game if no character
-            // with the given name exists, or load the corresponding
-            // game.
-            if (changed_name)
-            {
-                int i = _find_save(chars, input_string);
-                if (i == -1)
-                    menu.set_active_object(game_modes);
                 else
-                {
-                    menu.set_active_object(save_games);
-                    // save game ID is offset by NUM_GAME_TYPE
-                    save_games->set_active_item(NUM_GAME_TYPE + i);
-                }
+                    input_string.erase(input_string.size() - 1);
+                changed_name = true;
+                full_name = false;
             }
+        }
+
+        // Depending on whether the current name occurs
+        // in the saved games, update the active object.
+        // We want enter to start a new game if no character
+        // with the given name exists, or load the corresponding
+        // game.
+        if (changed_name)
+        {
+            int i = _find_save(chars, input_string);
+            if (i == -1)
+                menu.set_active_object(game_modes);
             else
             {
-                // Menu might have changed selection -- sync name.
-                type = menu.get_active_item()->get_id();
-                switch (type)
-                {
-                case GAME_TYPE_ARENA:
+                menu.set_active_object(save_games);
+                // save game ID is offset by NUM_GAME_TYPE
+                save_games->set_active_item(NUM_GAME_TYPE + i);
+            }
+        }
+        else
+        {
+            // Menu might have changed selection -- sync name.
+            // TODO: the mapping from menu item to game_type is alarmingly
+            // brittle here
+            startup_menu_game_type = menu.get_active_item()->get_id();
+            switch (startup_menu_game_type)
+            {
+            case GAME_TYPE_ARENA:
+                input_string = "";
+                break;
+            case GAME_TYPE_NORMAL:
+            case GAME_TYPE_TUTORIAL:
+            case GAME_TYPE_SPRINT:
+            case GAME_TYPE_HINTS:
+                // If a game type is chosen, the user expects
+                // to start a new game. Just blanking the name
+                // it it clashes for now.
+                if (_find_save(chars, input_string) != -1)
                     input_string = "";
-                    break;
-                case GAME_TYPE_NORMAL:
-                case GAME_TYPE_TUTORIAL:
-                case GAME_TYPE_SPRINT:
-                case GAME_TYPE_HINTS:
-                    // If a game type is chosen, the user expects
-                    // to start a new game. Just blanking the name
-                    // it it clashes for now.
-                    if (_find_save(chars, input_string) != -1)
-                        input_string = "";
-                    break;
-                case GAME_TYPE_HIGH_SCORES:
-                    break;
+                break;
+            case GAME_TYPE_HIGH_SCORES:
+                break;
 
-                case GAME_TYPE_INSTRUCTIONS:
-                    break;
+            case GAME_TYPE_INSTRUCTIONS:
+                break;
 
-                default:
-                    int save_number = type - NUM_GAME_TYPE;
-                    if (save_number < num_saves)
-                        input_string = chars.at(save_number).name;
-                    else // new game
-                        input_string = "";
-                    full_name = true;
-                    break;
-                }
+            default:
+                int save_number = startup_menu_game_type - NUM_GAME_TYPE;
+                if (save_number < num_saves)
+                    input_string = chars.at(save_number).name;
+                else // new game
+                    input_string = "";
+                full_name = true;
+                break;
             }
         }
-        // we had a significant action!
-        vector<MenuItem*> selected = menu.get_selected_items();
-        if (selected.empty())
+        input_text->set_text(input_string);
+    }
+    // we had a significant action!
+    vector<MenuItem*> selected = menu.get_selected_items();
+    menu.clear_selections();
+    if (selected.empty())
+    {
+        // Uninteresting action, poll a new key
+        return true;
+    }
+
+    int id = selected.at(0)->get_id();
+    switch (id)
+    {
+    case GAME_TYPE_NORMAL:
+    case GAME_TYPE_TUTORIAL:
+    case GAME_TYPE_SPRINT:
+    case GAME_TYPE_HINTS:
+        trim_string(input_string);
+        if (is_good_name(input_string, true))
         {
-            // Uninteresting action, poll a new key
-            continue;
+            ng_choice.type = static_cast<game_type>(id);
+            ng_choice.name = input_string;
+            return done = true;
         }
-
-        int id = selected.at(0)->get_id();
-        switch (id)
+        else
         {
-        case GAME_TYPE_NORMAL:
-        case GAME_TYPE_TUTORIAL:
-        case GAME_TYPE_SPRINT:
-        case GAME_TYPE_HINTS:
-            trim_string(input_string);
-            if (is_good_name(input_string, true, false))
-            {
-                ng_choice.type = static_cast<game_type>(id);
-                ng_choice.name = input_string;
-                return;
-            }
-            else
-            {
-                // bad name
-                cgotoxy(SCROLLER_MARGIN_X ,GAME_MODES_START_Y - 1);
-                clear_to_end_of_line();
-                textcolour(RED);
-                cprintf("That's a silly name");
-                // Don't make the next key re-enter the game.
-                menu.clear_selections();
-            }
-            continue;
-
-        case GAME_TYPE_ARENA:
-            ng_choice.type = GAME_TYPE_ARENA;
-            return;
-
-        case GAME_TYPE_INSTRUCTIONS:
-            list_commands();
-            // restart because help messes up CRTRegion
-            goto again;
-
-        case GAME_TYPE_HIGH_SCORES:
-            show_hiscore_table();
-            goto again;
-
-        default:
-            // It was a savegame instead
-            const int save_number = id - NUM_GAME_TYPE;
-            if (save_number < num_saves) // actual save
-            {
-                // Save the savegame character name
-                ng_choice.name = chars.at(save_number).name;
-                ng_choice.type = chars.at(save_number).saved_game_type;
-                ng_choice.filename = chars.at(save_number).filename;
-            }
-            else // "new game"
-            {
-                ng_choice.name = "";
-                ng_choice.type = GAME_TYPE_NORMAL;
-                ng_choice.filename = ""; // ?
-            }
-            return;
+            // bad name
+            descriptor->override_description("That's a silly name.");
+            // Don't make the next key re-enter the game.
+            menu.clear_selections();
         }
+        return true;
+
+    case GAME_TYPE_ARENA:
+        ng_choice.type = GAME_TYPE_ARENA;
+        return done = true;
+
+    case GAME_TYPE_INSTRUCTIONS:
+        show_help();
+        return true;
+
+    case GAME_TYPE_HIGH_SCORES:
+        show_hiscore_table();
+        return true;
+
+    default:
+        // It was a savegame instead
+        const int save_number = id - NUM_GAME_TYPE;
+        if (save_number < num_saves) // actual save
+        {
+            // Save the savegame character name
+            ng_choice.name = chars.at(save_number).name;
+            ng_choice.type = chars.at(save_number).saved_game_type;
+            ng_choice.filename = chars.at(save_number).filename;
+        }
+        else // "new game"
+        {
+            ng_choice.name = "";
+            ng_choice.type = GAME_TYPE_NORMAL;
+            ng_choice.filename = ""; // ?
+        }
+        return done = true;
+    }
+}
+
+/**
+ * Saves game mode and player name to ng_choice.
+ */
+static void _show_startup_menu(newgame_def& ng_choice,
+                               const newgame_def& defaults)
+{
+    unwind_bool no_more(crawl_state.show_more_prompt, false);
+
+#if defined(USE_TILE_LOCAL) && defined(TOUCH_UI)
+    wm->show_keyboard();
+#elif defined(USE_TILE_WEB)
+    tiles_crt_popup show_as_popup;
+#endif
+
+
+    auto startup_ui = make_shared<UIStartupMenu>(ng_choice, defaults);
+    auto popup = make_shared<ui::Popup>(startup_ui);
+
+    ui::run_layout(move(popup), startup_ui->done);
+
+    if (startup_ui->end_game)
+    {
+#ifdef USE_TILE_WEB
+        tiles.send_exit_reason("cancel");
+#endif
+        end(0);
     }
 }
 #endif
@@ -934,26 +1073,57 @@ again:
 static void _choose_arena_teams(newgame_def& choice,
                                 const newgame_def& defaults)
 {
+#ifdef USE_TILE_WEB
+    tiles_crt_popup show_as_popup;
+#endif
+
     if (!choice.arena_teams.empty())
         return;
-
     clear_message_store();
-    clrscr();
-
-    cprintf("Enter your choice of teams:\n");
-
-    cgotoxy(1, 4);
-    if (!defaults.arena_teams.empty())
-        cprintf("Enter - %s\n", defaults.arena_teams.c_str());
-    cprintf("\n");
-    cprintf("Examples:\n");
-    cprintf("  Sigmund v Jessica\n");
-    cprintf("  99 orc v the Royal Jelly\n");
-    cprintf("  20-headed hydra v 10 kobold ; scimitar ego:flaming\n");
-    cgotoxy(1, 2);
 
     char buf[80];
-    if (cancellable_get_line(buf, sizeof(buf)))
+    resumable_line_reader reader(buf, sizeof(buf));
+    bool done = false, cancel;
+    auto prompt_ui = make_shared<Text>();
+
+    prompt_ui->on(Widget::slots.event, [&](wm_event ev)  {
+        if (ev.type != WME_KEYDOWN)
+            return false;
+        int key = ev.key.keysym.sym;
+        key = reader.putkey(key);
+        if (key == -1)
+            return true;
+        cancel = !!key;
+        return done = true;
+    });
+
+    auto popup = make_shared<ui::Popup>(prompt_ui);
+    ui::push_layout(move(popup));
+    while (!done && !crawl_state.seen_hups)
+    {
+        string hlbuf = formatted_string(buf).to_colour_string();
+        if (hlbuf.find(" v ") != string::npos)
+            hlbuf = "<w>" + replace_all(hlbuf, " v ", "</w> v <w>") + "</w>";
+
+        formatted_string prompt;
+        prompt.cprintf("Enter your choice of teams:\n\n  ");
+        prompt += formatted_string::parse_string(hlbuf);
+
+        prompt.cprintf("\n\n");
+        if (!defaults.arena_teams.empty())
+            prompt.cprintf("Enter - %s\n", defaults.arena_teams.c_str());
+        prompt.cprintf("\n");
+        prompt.cprintf("Examples:\n");
+        prompt.cprintf("  Sigmund v Jessica\n");
+        prompt.cprintf("  99 orc v the Royal Jelly\n");
+        prompt.cprintf("  20-headed hydra v 10 kobold ; scimitar ego:flaming");
+        prompt_ui->set_text(prompt);
+
+        ui::pump_events();
+    }
+    ui::pop_layout();
+
+    if (cancel || crawl_state.seen_hups)
         game_ended(game_exit::abort);
     choice.arena_teams = buf;
     if (choice.arena_teams.empty())
@@ -987,6 +1157,8 @@ bool startup_step()
     crawl_state.type = choice.type;
 
     newgame_def defaults = read_startup_prefs();
+    if (crawl_state.default_startup_name.size() == 0)
+        crawl_state.default_startup_name = defaults.name;
 
     // Set the crawl_state gametype to the requested game type. This must
     // be done before looking for the savegame or the startup prefs file.
@@ -1003,15 +1175,18 @@ bool startup_step()
 
 #ifndef DGAMELAUNCH
 
+    if (recent_error_messages() && !Options.suppress_startup_errors)
+        replay_messages_during_startup();
+
     // startup
 
     // These conditions are ignored for tutorial or sprint, which always trigger
     // the relevant submenu. Arena never triggers a menu.
     const bool can_bypass_menu =
-            _exit_type_allows_menu_bypass(crawl_state.last_game_exit)
+            _exit_type_allows_menu_bypass(crawl_state.last_game_exit.exit_reason)
          && crawl_state.last_type != GAME_TYPE_ARENA
          && Options.name_bypasses_menu
-         && is_good_name(choice.name, false, false);
+         && is_good_name(choice.name, false);
 
     if (crawl_state.last_type == GAME_TYPE_TUTORIAL
         || crawl_state.last_type == GAME_TYPE_SPRINT)
@@ -1061,9 +1236,12 @@ bool startup_step()
     }
     else
     {
+        clear_message_store();
         setup_game(ng);
         newchar = true;
+        write_newgame_options_file(choice);
     }
+    crawl_state.default_startup_name = you.your_name;
 
     _post_init(newchar);
 
@@ -1072,7 +1250,7 @@ bool startup_step()
 
 
 
-static void _cio_init()
+void cio_init()
 {
     crawl_state.io_inited = true;
     console_startup();
