@@ -206,47 +206,19 @@ static bool _monster_random_space(const monster* mons, coord_def& target,
 
 void mons_relocated(monster* mons)
 {
-    // If the main body teleports get rid of the tentacles
     if (mons_is_tentacle_head(mons_base_type(*mons)))
-    {
-        for (monster_iterator mi; mi; ++mi)
-        {
-            if (mi->is_child_tentacle_of(mons))
-            {
-                for (monster_iterator connect; connect; ++connect)
-                {
-                    if (connect->is_child_tentacle_of(*mi))
-                        monster_die(*connect, KILL_RESET, -1, true, false);
-                }
-                monster_die(*mi, KILL_RESET, -1, true, false);
-            }
-        }
-    }
-    // If a tentacle/segment is relocated just kill the tentacle
+        destroy_tentacles(mons); // If the main body teleports get rid of the tentacles
     else if (mons->is_child_monster())
-    {
-        for (monster_iterator connect; connect; ++connect)
-        {
-            if (connect->is_child_tentacle_of(mons))
-                monster_die(*connect, KILL_RESET, -1, true, false);
-        }
-
-        monster_die(mons, KILL_RESET, -1, true, false);
-    }
-    // Kill an eldritch tentacle and all its segments.
+        destroy_tentacle(mons); // If a tentacle/segment is relocated just kill the tentacle
     else if (mons->type == MONS_ELDRITCH_TENTACLE
              || mons->type == MONS_ELDRITCH_TENTACLE_SEGMENT)
     {
+        // Kill an eldritch tentacle and all its segments.
         monster* tentacle = mons->type == MONS_ELDRITCH_TENTACLE
                             ? mons : monster_by_mid(mons->tentacle_connect);
 
-        for (monster_iterator mit; mit; ++mit)
-        {
-            if (mit->is_child_tentacle_of(tentacle))
-                monster_die(*mit, KILL_RESET, -1, true, false);
-        }
-
-        monster_die(tentacle, KILL_RESET, -1, true, false);
+        // this should take care of any tentacles
+        monster_die(*tentacle, KILL_RESET, -1, true, false);
     }
 
     mons->clear_clinging();
