@@ -425,6 +425,14 @@ bool feat_is_sealed(dungeon_feature_type feat)
            || feat == DNGN_SEALED_DOOR;
 }
 
+/** Is this feature runed, as in a runed door?
+ */
+bool cell_is_runed(const coord_def &p)
+{
+    // the orig_terrain call will check the actual terrain if there's no change
+    return orig_terrain(p) == DNGN_RUNED_DOOR;
+}
+
 /** Is this feature a type of statue, i.e., granite or an idol?
  */
 bool feat_is_statuelike(dungeon_feature_type feat)
@@ -695,9 +703,10 @@ int count_neighbours_with_func(const coord_def& c, bool (*checker)(dungeon_featu
 // For internal use by find_connected_identical only.
 static void _find_connected_identical(const coord_def &d,
                                       dungeon_feature_type ft,
-                                      set<coord_def>& out)
+                                      set<coord_def>& out,
+                                      bool known_only)
 {
-    if (grd(d) != ft)
+    if (grd(d) != ft || (known_only && !env.map_knowledge(d).known()))
         return;
 
     string prop = env.markers.property_at(d, MAT_ANY, "connected_exclude");
@@ -711,22 +720,22 @@ static void _find_connected_identical(const coord_def &d,
 
     if (out.insert(d).second)
     {
-        _find_connected_identical(coord_def(d.x+1, d.y), ft, out);
-        _find_connected_identical(coord_def(d.x-1, d.y), ft, out);
-        _find_connected_identical(coord_def(d.x, d.y+1), ft, out);
-        _find_connected_identical(coord_def(d.x, d.y-1), ft, out);
+        _find_connected_identical(coord_def(d.x+1, d.y), ft, out, known_only);
+        _find_connected_identical(coord_def(d.x-1, d.y), ft, out, known_only);
+        _find_connected_identical(coord_def(d.x, d.y+1), ft, out, known_only);
+        _find_connected_identical(coord_def(d.x, d.y-1), ft, out, known_only);
     }
 }
 
 // Find all connected cells containing ft, starting at d.
-void find_connected_identical(const coord_def &d, set<coord_def>& out)
+void find_connected_identical(const coord_def &d, set<coord_def>& out, bool known_only)
 {
     string prop = env.markers.property_at(d, MAT_ANY, "connected_exclude");
 
     if (!prop.empty())
         out.insert(d);
     else
-        _find_connected_identical(d, grd(d), out);
+        _find_connected_identical(d, grd(d), out, known_only);
 }
 
 void get_door_description(int door_size, const char** adjective, const char** noun)
@@ -769,7 +778,7 @@ static unique_ptr<map_mask_boolean> _slime_wall_precomputed_neighbour_mask;
 
 static void _precompute_slime_wall_neighbours()
 {
-    map_mask_boolean &mask(*_slime_wall_precomputed_neighbour_mask.get());
+    map_mask_boolean &mask(*_slime_wall_precomputed_neighbour_mask);
     for (rectangle_iterator ri(1); ri; ++ri)
     {
         if (grd(*ri) == DNGN_SLIMY_WALL)
@@ -786,7 +795,7 @@ unwind_slime_wall_precomputer::unwind_slime_wall_precomputer(bool docompute)
     if (!(env.level_state & LSTATE_SLIMY_WALL))
         return;
 
-    if (docompute && !_slime_wall_precomputed_neighbour_mask.get())
+    if (docompute && !_slime_wall_precomputed_neighbour_mask)
     {
         did_compute_mask = true;
         _slime_wall_precomputed_neighbour_mask.reset(
@@ -806,7 +815,7 @@ bool slime_wall_neighbour(const coord_def& c)
     if (!(env.level_state & LSTATE_SLIMY_WALL))
         return false;
 
-    if (_slime_wall_precomputed_neighbour_mask.get())
+    if (_slime_wall_precomputed_neighbour_mask)
         return (*_slime_wall_precomputed_neighbour_mask)(c);
 
     // Not using count_adjacent_slime_walls because the early return might
@@ -1301,7 +1310,6 @@ static void _announce_swap(coord_def pos1, coord_def pos2)
 
     const bool notable_seen1 = is_notable_terrain(feat1) && you.see_cell(pos1);
     const bool notable_seen2 = is_notable_terrain(feat2) && you.see_cell(pos2);
-    coord_def orig_pos, dest_pos;
 
     if (notable_seen1 && notable_seen2)
     {
