@@ -628,6 +628,8 @@ tileidx_t tilep_species_to_base_tile(int sp, int level)
         return TILEP_BASE_VINE_STALKER;
     case SP_BARACHI:
         return TILEP_BASE_BARACHI;
+    case SP_GNOLL:
+        return TILEP_BASE_GNOLL;
     default:
         return TILEP_BASE_HUMAN;
     }
@@ -671,15 +673,6 @@ void tilep_race_default(int sp, int level, dolls_data *doll)
         case SP_DEEP_ELF:
             hair = TILEP_HAIR_ELF_WHITE;
             break;
-        case SP_HILL_ORC:
-            hair = 0;
-            break;
-        case SP_KOBOLD:
-            hair = 0;
-            break;
-        case SP_MUMMY:
-            hair = 0;
-            break;
         case SP_TROLL:
             hair = TILEP_HAIR_TROLL;
             break;
@@ -697,15 +690,6 @@ void tilep_race_default(int sp, int level, dolls_data *doll)
             hair   = 0;
             break;
         }
-        case SP_MINOTAUR:
-            hair = 0;
-            break;
-        case SP_DEMONSPAWN:
-            hair = 0;
-            break;
-        case SP_GHOUL:
-            hair = 0;
-            break;
         case SP_MERFOLK:
             result = you.fishtail ? TILEP_BASE_MERFOLK_WATER
                                   : TILEP_BASE_MERFOLK;
@@ -725,7 +709,17 @@ void tilep_race_default(int sp, int level, dolls_data *doll)
             hair = 0;
             beard = TILEP_BEARD_MEDIUM_GREEN;
             break;
+        case SP_MINOTAUR:
+        case SP_DEMONSPAWN:
+        case SP_GHOUL:
+        case SP_HILL_ORC:
+        case SP_KOBOLD:
+        case SP_MUMMY:
         case SP_FORMICID:
+        case SP_BARACHI:
+        case SP_GNOLL:
+        case SP_GARGOYLE:
+        case SP_VINE_STALKER:
             hair = 0;
             break;
         default:
