@@ -21,9 +21,9 @@
 #include "mon-util.h"
 #include "options.h"
 #include "output.h"
-#include "process-desc.h"
 #include "rot.h"
 #include "spl-book.h"
+#include "stringutil.h"
 #include "tile-inventory-flags.h"
 #include "tiledef-dngn.h"
 #include "tiledef-icons.h"
@@ -598,11 +598,7 @@ bool InventoryRegion::update_alt_text(string &alt)
     else
         get_item_desc(*item, inf);
 
-    alt_desc_proc proc(crawl_view.msgsz.x, crawl_view.msgsz.y);
-    process_description<alt_desc_proc>(proc, inf);
-
-    proc.get_string(alt);
-
+    alt = process_description(inf);
     return true;
 }
 
@@ -631,7 +627,7 @@ void InventoryRegion::draw_tag()
 
 void InventoryRegion::activate()
 {
-    if (inv_count() < 1)
+    if (inv_count() < 1 && you.num_turns > 0)
     {
         canned_msg(MSG_NOTHING_CARRIED);
         flush_prev_message();
