@@ -312,7 +312,8 @@ void tile_init_flavour()
     vector<unsigned int> output;
     {
         domino::DominoSet<domino::EdgeDomino> dominoes(domino::cohen_set, 8);
-        uint64_t seed[] = { get_uint64(RNG_UI), get_uint64(RNG_UI) };
+        uint64_t seed[] = { static_cast<uint64_t>(you.where_are_you ^ you.game_seed),
+            static_cast<uint64_t>(you.depth) };
         PcgRNG rng(seed, ARRAYSZ(seed));
         dominoes.Generate(X_WIDTH, Y_WIDTH, output, rng);
     }
@@ -361,12 +362,13 @@ static void _get_depths_wall_tiles_by_depth(int depth, vector<tileidx_t>& t)
         t.push_back(TILE_WALL_BRICK_DARK_6_TORCH);  // ...and on Depths:$
 }
 
-static int _find_variants(tileidx_t idx, int variant, map<tileidx_t, int> &out)
+static int _find_variants(tileidx_t idx, int variant, vector<int> &out)
 {
     const int count = tile_dngn_count(idx);
+    out.reserve(count);
     if (count == 1)
     {
-        out[idx] = 1;
+        out.push_back(1);
         return 1;
     }
 
@@ -380,17 +382,17 @@ static int _find_variants(tileidx_t idx, int variant, map<tileidx_t, int> &out)
         {
             int weight = curr_prob - last_prob;
             total += weight;
-            out[idx + i] = weight;
+            out.push_back(weight);
         }
+        else
+            out.push_back(0);
     }
-    if (out.empty())
+    if (!total)
     {
-        out[idx] = tile_dngn_probs(idx);
+        out.clear();
+        out.push_back(tile_dngn_probs(idx));
         for (int i = 1; i < count; ++i)
-        {
-            out[idx + i] = tile_dngn_probs(idx + i)
-                           - tile_dngn_probs(idx + i - 1);
-        }
+            out.push_back(tile_dngn_probs(idx + i) - tile_dngn_probs(idx + i - 1));
         return tile_dngn_probs(idx + count - 1);
     }
     return total;
@@ -399,17 +401,18 @@ static int _find_variants(tileidx_t idx, int variant, map<tileidx_t, int> &out)
 tileidx_t pick_dngn_tile(tileidx_t idx, int value, int domino)
 {
     ASSERT_LESS(idx, TILE_DNGN_MAX);
-    map<tileidx_t, int> choices;
-    int total = _find_variants(idx, domino, choices);
-    if (choices.size() == 1)
-        return choices.begin()->first;
-    int rand  = value % total;
+    static vector<int> weights;
+    weights.clear();
 
-    for (const auto& elem : choices)
+    int total = _find_variants(idx, domino, weights);
+    if (weights.size() == 1)
+        return idx;
+    int rand = value % total;
+
+    for (size_t i = 0; i < weights.size(); ++i)
     {
-        rand -= elem.second;
-        if (rand < 0)
-            return elem.first;
+        rand -= weights[i];
+        if (rand < 0) return idx + i;
     }
 
     return idx;
@@ -461,8 +464,8 @@ void tile_init_flavour(const coord_def &gc, const int domino)
     uint32_t seed = you.birth_time + you.where_are_you +
         (you.depth << 8) + (gc.x << 16) + (gc.y << 24);
 
-    int rand1 = hash_rand(INT_MAX, seed, 0);
-    int rand2 = hash_rand(INT_MAX, seed, 1);
+    int rand1 = hash_with_seed(INT_MAX, seed, 0);
+    int rand2 = hash_with_seed(INT_MAX, seed, 1);
 
     if (!env.tile_flv(gc).floor)
     {
@@ -551,7 +554,7 @@ void tile_init_flavour(const coord_def &gc, const int domino)
             env.tile_flv(gc).special = 0;
     }
     else if (!env.tile_flv(gc).special)
-        env.tile_flv(gc).special = hash_rand(256, seed, 10);
+        env.tile_flv(gc).special = hash_with_seed(256, seed, 10);
 }
 
 enum SpecialIdx
@@ -861,7 +864,10 @@ void tile_forget_map(const coord_def &gc)
     env.tile_bk_fg(gc) = 0;
     env.tile_bk_bg(gc) = 0;
     env.tile_bk_cloud(gc) = 0;
-    tiles.update_minimap(gc);
+    // This may have changed the explore horizon, so update adjacent minimap
+    // squares as well.
+    for (adjacent_iterator ai(gc, false); ai; ++ai)
+        tiles.update_minimap(*ai);
 }
 
 static void _tile_place_item(const coord_def &gc, const item_info &item,
