@@ -33,6 +33,7 @@
 #include "tileview.h"
 #include "tiles-build-specific.h"
 #include "travel.h"
+#include "ui.h"
 #include "unicode.h"
 #include "view.h"
 #include "viewchar.h"
@@ -107,29 +108,6 @@ bool travel_colour_override(const coord_def& p)
         return false;
 }
 
-static bool _is_explore_horizon(const coord_def& c)
-{
-    if (env.map_knowledge(c).feat() != DNGN_UNSEEN)
-        return false;
-
-    // Note: c might be on map edge, walkable squares not really.
-    for (adjacent_iterator ai(c); ai; ++ai)
-        if (in_bounds(*ai))
-        {
-            dungeon_feature_type feat = env.map_knowledge(*ai).feat();
-            if (feat != DNGN_UNSEEN
-                && !feat_is_solid(feat)
-                && !feat_is_door(feat))
-            {
-                return true;
-            }
-        }
-
-    return false;
-}
-#endif
-
-#ifndef USE_TILE_LOCAL
 static char32_t _get_sightmap_char(dungeon_feature_type feat)
 {
     return get_feature_def(feat).symbol();
@@ -361,7 +339,7 @@ static void _draw_level_map(int start_x, int start_y, bool travel_mode,
 
                 const show_class show = get_cell_show_class(env.map_knowledge(c));
 
-                if (show == SH_NOTHING && _is_explore_horizon(c))
+                if (show == SH_NOTHING && is_explore_horizon(c))
                 {
                     const feature_def& fd = get_feature_def(DNGN_EXPLORE_HORIZON);
                     cell->glyph = fd.symbol();
@@ -593,8 +571,8 @@ static level_pos _stair_dest(const coord_def& p, command_type dir)
 
 static void _unforget_map()
 {
-    ASSERT(env.map_forgotten.get());
-    MapKnowledge &old(*env.map_forgotten.get());
+    ASSERT(env.map_forgotten);
+    MapKnowledge &old(*env.map_forgotten);
 
     for (rectangle_iterator ri(0); ri; ++ri)
         if (!env.map_knowledge(*ri).seen() && old(*ri).seen())
@@ -609,7 +587,7 @@ static void _unforget_map()
         }
 }
 
-static void _forget_map()
+static void _forget_map(bool wizard_forget = false)
 {
     for (rectangle_iterator ri(0); ri; ++ri)
     {
@@ -617,9 +595,16 @@ static void _forget_map()
         // don't touch squares we can currently see
         if (flags & MAP_VISIBLE_FLAG)
             continue;
-        // squares we've seen in the past, pretend we've mapped instead
-        if (flags & MAP_SEEN_FLAG)
+        if (wizard_forget)
         {
+            env.map_knowledge(*ri).clear();
+#ifdef USE_TILE
+            tile_forget_map(*ri);
+#endif
+        }
+        else if (flags & MAP_SEEN_FLAG)
+        {
+            // squares we've seen in the past, pretend we've mapped instead
             flags |= MAP_MAGIC_MAPPED_FLAG;
             flags &= ~MAP_SEEN_FLAG;
         }
@@ -645,8 +630,10 @@ bool show_map(level_pos &lpos,
     tiles.do_map_display();
 #endif
 
+#ifdef USE_TILE
+    ui::cutoff_point ui_cutoff_point;
+#endif
 #ifdef USE_TILE_WEB
-    tiles_crt_control crt(false);
     tiles_ui_control ui(UI_VIEW_MAP);
 #endif
 
@@ -901,10 +888,29 @@ bool show_map(level_pos &lpos,
                 clear_map_or_travel_trail();
                 break;
 
+#ifdef WIZARD
+            case CMD_MAP_WIZARD_FORGET:
+                {
+                    // this doesn't seem useful outside of debugging and may
+                    // be buggy in unexpected ways, so wizmode-only. (Though
+                    // it doesn't leak information or anything.)
+                    if (!you.wizard)
+                        break;
+                    if (env.map_forgotten)
+                        _unforget_map();
+                    MapKnowledge *old = new MapKnowledge(env.map_knowledge);
+                    // completely wipe out map
+                    _forget_map(true);
+                    env.map_forgotten.reset(old);
+                    mpr("Level map wiped.");
+                    break;
+                }
+#endif
+
             case CMD_MAP_FORGET:
                 {
                     // Merge it with already forgotten data first.
-                    if (env.map_forgotten.get())
+                    if (env.map_forgotten)
                         _unforget_map();
                     MapKnowledge *old = new MapKnowledge(env.map_knowledge);
                     _forget_map();
@@ -914,7 +920,7 @@ bool show_map(level_pos &lpos,
                 break;
 
             case CMD_MAP_UNFORGET:
-                if (env.map_forgotten.get())
+                if (env.map_forgotten)
                 {
                     _unforget_map();
                     env.map_forgotten.reset();
@@ -1180,8 +1186,19 @@ bool show_map(level_pos &lpos,
                 {
                     if (you.travel_x > 0 && you.travel_y > 0)
                     {
-                        move_x = you.travel_x - lpos.pos.x;
-                        move_y = you.travel_y - lpos.pos.y;
+                        if (you.travel_z == level_id::current())
+                        {
+                            move_x = you.travel_x - lpos.pos.x;
+                            move_y = you.travel_y - lpos.pos.y;
+                        }
+                        else if (allow_offlevel && you.travel_z.is_valid()
+                                        && is_existing_level(you.travel_z))
+                        {
+                            // previous travel target is offlevel
+                            lpos = level_pos(you.travel_z,
+                                        coord_def(you.travel_x, you.travel_y));
+                            los_changed();
+                        }
                     }
                 }
                 else
