@@ -23,6 +23,9 @@ logging_config = {
 }
 
 password_db = "./webserver/passwd.db3"
+# Uncomment and change if you want this db somewhere separate from the
+# password_db location.
+#settings_db = "./webserver/user_settings.db3"
 
 static_path = "./webserver/static"
 template_path = "./webserver/templates/"
@@ -68,19 +71,6 @@ games = OrderedDict([
         morgue_url = None,
         send_json_options = True,
         options = ["-sprint"])),
-    ("zd-web-trunk", dict(
-        name = "Zot Defense trunk",
-        crawl_binary = "./crawl",
-        rcfile_path = "./rcs/",
-        macro_path = "./rcs/",
-        morgue_path = "./rcs/%n",
-        inprogress_path = "./rcs/running",
-        ttyrec_path = "./rcs/ttyrecs/%n",
-        socket_path = "./rcs",
-        client_path = "./webserver/game_data/",
-        morgue_url = None,
-        send_json_options = True,
-        options = ["-zotdef"])),
     ("tut-web-trunk", dict(
         name = "Tutorial trunk",
         crawl_binary = "./crawl",
