@@ -65,7 +65,6 @@ static void _make_all_books()
 
         item_def book(mitm[thing]);
 
-        mark_had_book(book);
         set_ident_flags(book, ISFLAG_KNOW_TYPE);
         set_ident_flags(book, ISFLAG_IDENT_MASK);
 
@@ -1027,7 +1026,7 @@ static void _debug_acquirement_stats(FILE *ostat)
             if (spell_rarity(spell) == -1)
                 continue;
 
-            const bool seen = you.seen_spell[spell];
+            const bool seen = you.spell_library[spell];
 
             const spschools_type disciplines = get_spell_disciplines(spell);
             for (int d = 0; d <= SPTYP_LAST_EXPONENT; ++d)
