@@ -713,24 +713,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     6
 },
 
-{
-    ZAP_ENSLAVE_SOUL,
-    "",
-    100,
-    nullptr,
-    nullptr,
-    nullptr,
-    nullptr,
-    BLACK,
-    true,
-    BEAM_ENSLAVE_SOUL,
-    NUM_DCHAR_TYPES,
-    false,
-    false,
-    false,
-    0
-},
-
 {
     ZAP_AGONY,
     "agony",
@@ -1197,7 +1179,7 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     nullptr,
     new dicedef_calculator<2, 10, 1, 17>,
     new tohit_calculator<16, 1, 40>,
-    LIGHTCYAN, // match slug's own colour
+    LIGHTCYAN,
     false,
     BEAM_ELECTRICITY,
     DCHAR_FIRED_ZAP,
@@ -1215,7 +1197,7 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     nullptr,
     new dicedef_calculator<2, 9, 1, 11>,
     new tohit_calculator<17, 1, 25>,
-    WHITE, // match slug's own colour
+    WHITE,
     false,
     BEAM_COLD,
     DCHAR_FIRED_ZAP,
@@ -1225,6 +1207,24 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     0
 },
 
+{
+    ZAP_HARPOON_SHOT,
+    "harpoon shot",
+    50,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<2, 7, 1, 20>,
+    new tohit_calculator<17, 1, 25>,
+    CYAN,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_MISSILE,
+    true,
+    false,
+    false,
+    1
+},
+
 _mon_hex_zap(ZAP_DIMENSION_ANCHOR, BEAM_DIMENSION_ANCHOR),
 _mon_hex_zap(ZAP_VULNERABILITY, BEAM_VULNERABILITY),
 _mon_hex_zap(ZAP_SENTINEL_MARK, BEAM_SENTINEL_MARK),
