@@ -52,7 +52,6 @@ enum zap_type
     ZAP_POISON_ARROW,
     ZAP_BREATHE_STICKY_FLAME,
     ZAP_PETRIFY,
-    ZAP_ENSLAVE_SOUL,
     ZAP_PORKALATOR,
     ZAP_SLEEP,
     ZAP_PRIMAL_WAVE,
@@ -86,5 +85,6 @@ enum zap_type
     ZAP_FIRE_STORM,
     ZAP_MALMUTATE,
     ZAP_VILE_CLUTCH,
+    ZAP_HARPOON_SHOT,
     NUM_ZAPS
 };
